@@ -1,39 +1,7 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is mozilla.org code.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 1998
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either of the GNU General Public License Version 2 or later (the "GPL"),
- * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 #ifndef nsIDocumentObserver_h___
 #define nsIDocumentObserver_h___
 
@@ -53,7 +21,7 @@ class nsIParser;
 { 0x900bc4bc, 0x8b6c, 0x4cba, \
  { 0x82, 0xfa, 0x56, 0x8a, 0x80, 0xff, 0xfd, 0x3e } }
 
-typedef PRUint32 nsUpdateType;
+typedef uint32_t nsUpdateType;
 
 #define UPDATE_CONTENT_MODEL 0x00000001
 #define UPDATE_STYLE         0x00000002
@@ -132,7 +100,7 @@ public:
    */
   NS_IMETHOD_(void) StyleSheetAdded(nsIDocument *aDocument,
                                nsIStyleSheet* aStyleSheet,
-                               PRBool aDocumentSheet) = 0;
+                               bool aDocumentSheet) = 0;
 
   /**
    * A StyleSheet has just been removed from the document.  This
@@ -147,7 +115,7 @@ public:
    */
   NS_IMETHOD_(void) StyleSheetRemoved(nsIDocument *aDocument,
                                  nsIStyleSheet* aStyleSheet,
-                                 PRBool aDocumentSheet) = 0;
+                                 bool aDocumentSheet) = 0;
   
   /**
    * A StyleSheet has just changed its applicable state.
@@ -158,12 +126,12 @@ public:
    *
    * @param aDocument The document being observed
    * @param aStyleSheet the StyleSheet that has changed state
-   * @param aApplicable PR_TRUE if the sheet is applicable, PR_FALSE if
+   * @param aApplicable true if the sheet is applicable, false if
    *        it is not applicable
    */
   NS_IMETHOD_(void) StyleSheetApplicableStateChanged(nsIDocument *aDocument,
                                                 nsIStyleSheet* aStyleSheet,
-                                                PRBool aApplicable) = 0;
+                                                bool aApplicable) = 0;
 
   /**
    * A StyleRule has just been modified within a style sheet.
@@ -230,7 +198,7 @@ public:
   NS_IMETHOD_(void) BindToDocument(nsIDocument *aDocument,
                               nsIContent *aContent) = 0;
 
-  NS_IMETHOD DoneAddingChildren(nsIContent *aContent, PRBool aHaveNotified, nsIParser *aParser) = 0;
+  NS_IMETHOD_(void) AttemptToExecuteScript(nsIContent *aContent, nsIParser *aParser, bool *aBlock) = 0;
 #endif
 };
 
@@ -261,17 +229,17 @@ NS_DEFINE_STATIC_IID_ACCESSOR(nsIDocumentObserver, NS_IDOCUMENT_OBSERVER_IID)
 #define NS_DECL_NSIDOCUMENTOBSERVER_STYLESHEETADDED                          \
     NS_IMETHOD_(void) StyleSheetAdded(nsIDocument* aDocument,                \
                                  nsIStyleSheet* aStyleSheet,                 \
-                                 PRBool aDocumentSheet);
+                                 bool aDocumentSheet);
 
 #define NS_DECL_NSIDOCUMENTOBSERVER_STYLESHEETREMOVED                        \
     NS_IMETHOD_(void) StyleSheetRemoved(nsIDocument* aDocument,              \
                                    nsIStyleSheet* aStyleSheet,               \
-                                   PRBool aDocumentSheet);
+                                   bool aDocumentSheet);
 
 #define NS_DECL_NSIDOCUMENTOBSERVER_STYLESHEETAPPLICABLESTATECHANGED         \
     NS_IMETHOD_(void) StyleSheetApplicableStateChanged(nsIDocument* aDocument, \
                                                   nsIStyleSheet* aStyleSheet,\
-                                                  PRBool aApplicable);
+                                                  bool aApplicable);
 
 #define NS_DECL_NSIDOCUMENTOBSERVER_STYLERULECHANGED                         \
     NS_IMETHOD_(void) StyleRuleChanged(nsIDocument* aDocument,               \
@@ -295,15 +263,15 @@ NS_DEFINE_STATIC_IID_ACCESSOR(nsIDocumentObserver, NS_IDOCUMENT_OBSERVER_IID)
     NS_IMETHOD_(void) BindToDocument(nsIDocument *aDocument,                 \
                                      nsIContent *aContent);
 
-#define NS_DECL_NSIDOCUMENTOBSERVER_DONEADDINGCHILDREN                       \
-    NS_IMETHOD DoneAddingChildren(nsIContent *aContent,                      \
-                                  PRBool aHaveNotified,                      \
-                                  nsIParser *aParser);
+#define NS_DECL_NSIDOCUMENTOBSERVER_ATTEMPTTOEXECUTESCRIPT                   \
+  NS_IMETHOD_(void) AttemptToExecuteScript(nsIContent *aContent,             \
+                                  nsIParser *aParser,                        \
+                                  bool *aBlock);
 
 #else
 
 #define NS_DECL_NSIDOCUMENTOBSERVER_BINDTODOCUMENT
-#define NS_DECL_NSIDOCUMENTOBSERVER_DONEADDINGCHILDREN
+#define NS_DECL_NSIDOCUMENTOBSERVER_ATTEMPTTOEXECUTESCRIPT
 
 #endif
 
@@ -321,7 +289,7 @@ NS_DEFINE_STATIC_IID_ACCESSOR(nsIDocumentObserver, NS_IDOCUMENT_OBSERVER_IID)
     NS_DECL_NSIDOCUMENTOBSERVER_STYLERULEADDED                               \
     NS_DECL_NSIDOCUMENTOBSERVER_STYLERULEREMOVED                             \
     NS_DECL_NSIDOCUMENTOBSERVER_BINDTODOCUMENT                               \
-    NS_DECL_NSIDOCUMENTOBSERVER_DONEADDINGCHILDREN                           \
+    NS_DECL_NSIDOCUMENTOBSERVER_ATTEMPTTOEXECUTESCRIPT                       \
     NS_DECL_NSIMUTATIONOBSERVER
 
 
@@ -368,11 +336,10 @@ void _class::BindToDocument(nsIDocument *aDocument,                       \
                             nsIContent *aContent)                         \
 { \
 } \
-nsresult _class::DoneAddingChildren(nsIContent *aContent,                 \
-                                    PRBool aNotified,                     \
-                                    nsIParser *aParser)                   \
+void _class::AttemptToExecuteScript(nsIContent *aContent,                 \
+                                    nsIParser *aParser,                   \
+                                    bool *aBlock)                         \
 { \
- return NS_OK; \
 }
 
 #else
@@ -385,19 +352,19 @@ nsresult _class::DoneAddingChildren(nsIContent *aContent,                 \
 void                                                                      \
 _class::StyleSheetAdded(nsIDocument* aDocument,                           \
                         nsIStyleSheet* aStyleSheet,                       \
-                        PRBool aDocumentSheet)                            \
+                        bool aDocumentSheet)                            \
 {                                                                         \
 }                                                                         \
 void                                                                      \
 _class::StyleSheetRemoved(nsIDocument* aDocument,                         \
                           nsIStyleSheet* aStyleSheet,                     \
-                          PRBool aDocumentSheet)                          \
+                          bool aDocumentSheet)                          \
 {                                                                         \
 }                                                                         \
 void                                                                      \
 _class::StyleSheetApplicableStateChanged(nsIDocument* aDocument,          \
                                          nsIStyleSheet* aStyleSheet,      \
-                                         PRBool aApplicable)              \
+                                         bool aApplicable)              \
 {                                                                         \
 }                                                                         \
 void                                                                      \
