@@ -1,43 +1,8 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /* vim: set ts=2 sw=2 et tw=79: */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is mozilla.org code.
- *
- * The Initial Developer of the Original Code is
- * Mozilla Corporation.
- * Portions created by the Initial Developer are Copyright (C) 2007
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Dave Camp <dcamp@mozilla.com>
- *   Robert Sayre <sayrer@gmail.com>
- *   Nils Maier <maierman@web.de>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "jsapi.h"
 #include "jsdbgapi.h"
@@ -56,6 +21,7 @@
 #include "nsCRTGlue.h"
 #include "nsAutoPtr.h"
 #include "nsIScriptSecurityManager.h"
+#include <algorithm>
 
 static const char kXPConnectServiceCID[] = "@mozilla.org/js/xpc/XPConnect;1";
 
@@ -82,27 +48,30 @@ enum DeprecationWarning { EncodeWarning, DecodeWarning };
 static nsresult
 WarnDeprecatedMethod(DeprecationWarning warning)
 {
-  return nsContentUtils::ReportToConsole(nsContentUtils::eDOM_PROPERTIES,
+  return nsContentUtils::ReportToConsole(nsIScriptError::warningFlag,
+                                         "DOM Core", nullptr,
+                                         nsContentUtils::eDOM_PROPERTIES,
                                          warning == EncodeWarning
                                          ? "nsIJSONEncodeDeprecatedWarning"
-                                         : "nsIJSONDecodeDeprecatedWarning",
-                                         nsnull, 0,
-                                         nsnull,
-                                         EmptyString(), 0, 0,
-                                         nsIScriptError::warningFlag,
-                                         "DOM Core");
+                                         : "nsIJSONDecodeDeprecatedWarning");
 }
 
 NS_IMETHODIMP
-nsJSON::Encode(nsAString &aJSON)
+nsJSON::Encode(const JS::Value& aValue, JSContext* cx, uint8_t aArgc, nsAString &aJSON)
 {
   // This function should only be called from JS.
   nsresult rv = WarnDeprecatedMethod(EncodeWarning);
   if (NS_FAILED(rv))
     return rv;
 
+  if (aArgc == 0) {
+    aJSON.Truncate();
+    aJSON.SetIsVoid(true);
+    return NS_OK;
+  }
+
   nsJSONWriter writer;
-  rv = EncodeInternal(&writer);
+  rv = EncodeInternal(cx, aValue, &writer);
 
   // FIXME: bug 408838. Get exception types sorted out
   if (NS_SUCCEEDED(rv) || rv == NS_ERROR_INVALID_ARG) {
@@ -110,7 +79,7 @@ nsJSON::Encode(nsAString &aJSON)
     // if we didn't consume anything, it's not JSON, so return null
     if (!writer.DidWrite()) {
       aJSON.Truncate();
-      aJSON.SetIsVoid(PR_TRUE);
+      aJSON.SetIsVoid(true);
     } else {
       writer.FlushBuffer();
       aJSON.Append(writer.mOutputString);
@@ -139,7 +108,10 @@ static nsresult CheckCharset(const char* aCharset)
 NS_IMETHODIMP
 nsJSON::EncodeToStream(nsIOutputStream *aStream,
                        const char* aCharset,
-                       const PRBool aWriteBOM)
+                       const bool aWriteBOM,
+                       const JS::Value& val,
+                       JSContext* cx,
+                       uint8_t aArgc)
 {
   // This function should only be called from JS.
   NS_ENSURE_ARG(aStream);
@@ -159,7 +131,7 @@ nsJSON::EncodeToStream(nsIOutputStream *aStream,
   //  aStream = bufferedStream;
   //}
 
-  PRUint32 ignored;
+  uint32_t ignored;
   if (aWriteBOM) {
     if (strcmp(aCharset, "UTF-8") == 0)
       rv = aStream->Write(UTF8BOM, 3, &ignored);
@@ -174,7 +146,11 @@ nsJSON::EncodeToStream(nsIOutputStream *aStream,
   rv = writer.SetCharset(aCharset);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  rv = EncodeInternal(&writer);
+  if (aArgc == 0) {
+    return NS_OK;
+  }
+
+  rv = EncodeInternal(cx, val, &writer);
   NS_ENSURE_SUCCESS(rv, rv);
 
   rv = bufferedStream->Flush();
@@ -183,10 +159,10 @@ nsJSON::EncodeToStream(nsIOutputStream *aStream,
 }
 
 static JSBool
-WriteCallback(const jschar *buf, uint32 len, void *data)
+WriteCallback(const jschar *buf, uint32_t len, void *data)
 {
   nsJSONWriter *writer = static_cast<nsJSONWriter*>(data);
-  nsresult rv =  writer->Write((const PRUnichar*)buf, (PRUint32)len);
+  nsresult rv =  writer->Write((const PRUnichar*)buf, (uint32_t)len);
   if (NS_FAILED(rv))
     return JS_FALSE;
 
@@ -194,40 +170,20 @@ WriteCallback(const jschar *buf, uint32 len, void *data)
 }
 
 NS_IMETHODIMP
-nsJSON::EncodeFromJSVal(jsval *value, JSContext *cx, nsAString &result)
+nsJSON::EncodeFromJSVal(JS::Value *value, JSContext *cx, nsAString &result)
 {
   result.Truncate();
 
   // Begin a new request
   JSAutoRequest ar(cx);
 
-  JSAutoEnterCompartment ac;
-  JSObject *obj;
-  nsIScriptSecurityManager *ssm = nsnull;
-  if (JSVAL_IS_OBJECT(*value) && (obj = JSVAL_TO_OBJECT(*value))) {
-    if (!ac.enter(cx, obj)) {
-      return NS_ERROR_FAILURE;
-    }
-
-    nsCOMPtr<nsIPrincipal> principal;
-    ssm = nsContentUtils::GetSecurityManager();
-    nsresult rv = ssm->GetObjectPrincipal(cx, obj, getter_AddRefs(principal));
-    NS_ENSURE_SUCCESS(rv, rv);
-
-    JSStackFrame *fp = nsnull;
-    rv = ssm->PushContextPrincipal(cx, JS_FrameIterator(cx, &fp), principal);
-    NS_ENSURE_SUCCESS(rv, rv);
+  mozilla::Maybe<JSAutoCompartment> ac;
+  if (value->isObject()) {
+    ac.construct(cx, &value->toObject());
   }
 
   nsJSONWriter writer;
-  JSBool ok = JS_Stringify(cx, value, NULL, JSVAL_NULL,
-                           WriteCallback, &writer);
-
-  if (ssm) {
-    ssm->PopContextPrincipal(cx);
-  }
-
-  if (!ok) {
+  if (!JS_Stringify(cx, value, NULL, JSVAL_NULL, WriteCallback, &writer)) {
     return NS_ERROR_XPC_BAD_CONVERT_JS;
   }
 
@@ -238,52 +194,19 @@ nsJSON::EncodeFromJSVal(jsval *value, JSContext *cx, nsAString &result)
 }
 
 nsresult
-nsJSON::EncodeInternal(nsJSONWriter *writer)
+nsJSON::EncodeInternal(JSContext* cx, const JS::Value& aValue, nsJSONWriter* writer)
 {
-  nsresult rv;
-  nsIXPConnect *xpc = nsContentUtils::XPConnect();
-  if (!xpc)
-    return NS_ERROR_FAILURE;
-
-  // Now fish for the JS argument. If it's a call to encode, we'll
-  // want the first argument. If it's a call to encodeToStream,
-  // we'll want the forth.
-  const PRUint32 firstArg = writer->mStream ? 3 : 0;
-
-  nsAXPCNativeCallContext *cc = nsnull;
-  rv = xpc->GetCurrentNativeCallContext(&cc);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  PRUint32 argc = 0;
-  rv = cc->GetArgc(&argc);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  // If the argument wasn't provided, there's nothing to serialize.
-  if (argc <= firstArg)
-    return NS_OK;
-
-  JSContext *cx = nsnull;
-  rv = cc->GetJSContext(&cx);
-  NS_ENSURE_SUCCESS(rv, rv);
-
   JSAutoRequest ar(cx);
 
-  // Get the object we're going to serialize.
-  jsval *argv = nsnull;
-  rv = cc->GetArgvPtr(&argv);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  // If the argument wasn't provided, there's nothing to serialize.
-  if (argc <= firstArg)
-    return NS_ERROR_INVALID_ARG;
-
-  jsval *vp = &argv[firstArg];
-  
   // Backward compatibility:
   // nsIJSON does not allow to serialize anything other than objects
-  JSObject *obj;
-  if (!JSVAL_IS_OBJECT(*vp) || !(obj = JSVAL_TO_OBJECT(*vp)))
+  if (!aValue.isObject()) {
     return NS_ERROR_INVALID_ARG;
+  }
+
+  JSObject* obj = &aValue.toObject();
+
+  JS::Value val = aValue;
 
   /* Backward compatibility:
    * Manually call toJSON if implemented by the object and check that
@@ -295,9 +218,8 @@ nsJSON::EncodeInternal(nsJSONWriter *writer)
   if (JS_GetMethod(cx, obj, "toJSON", NULL, &toJSON) &&
       !JSVAL_IS_PRIMITIVE(toJSON) &&
       JS_ObjectIsCallable(cx, JSVAL_TO_OBJECT(toJSON))) {
-
     // If toJSON is implemented, it must not throw
-    if (!JS_CallFunctionValue(cx, obj, toJSON, 0, NULL, vp)) {
+    if (!JS_CallFunctionValue(cx, obj, toJSON, 0, NULL, &val)) {
       if (JS_IsExceptionPending(cx))
         // passing NS_OK will throw the pending exception
         return NS_OK;
@@ -308,7 +230,7 @@ nsJSON::EncodeInternal(nsJSONWriter *writer)
 
     // Backward compatibility:
     // nsIJSON does not allow to serialize anything other than objects
-    if (JSVAL_IS_PRIMITIVE(*vp))
+    if (JSVAL_IS_PRIMITIVE(val))
       return NS_ERROR_INVALID_ARG;
   }
   // GetMethod may have thrown
@@ -317,32 +239,32 @@ nsJSON::EncodeInternal(nsJSONWriter *writer)
     return NS_OK;
 
   // Backward compatibility:
-  // function/xml shall not pass, just "plain" objects and arrays
-  JSType type = JS_TypeOfValue(cx, *vp);
-  if (type == JSTYPE_FUNCTION || type == JSTYPE_XML)
+  // function shall not pass, just "plain" objects and arrays
+  JSType type = JS_TypeOfValue(cx, val);
+  if (type == JSTYPE_FUNCTION)
     return NS_ERROR_INVALID_ARG;
 
   // We're good now; try to stringify
-  if (!JS_Stringify(cx, vp, NULL, JSVAL_NULL, WriteCallback, writer))
+  if (!JS_Stringify(cx, &val, NULL, JSVAL_NULL, WriteCallback, writer))
     return NS_ERROR_FAILURE;
 
   return NS_OK;
 }
 
 
-nsJSONWriter::nsJSONWriter() : mStream(nsnull),
-                               mBuffer(nsnull),
+nsJSONWriter::nsJSONWriter() : mStream(nullptr),
+                               mBuffer(nullptr),
                                mBufferCount(0),
-                               mDidWrite(PR_FALSE),
-                               mEncoder(nsnull)
+                               mDidWrite(false),
+                               mEncoder(nullptr)
 {
 }
 
 nsJSONWriter::nsJSONWriter(nsIOutputStream *aStream) : mStream(aStream),
-                                                       mBuffer(nsnull),
+                                                       mBuffer(nullptr),
                                                        mBufferCount(0),
-                                                       mDidWrite(PR_FALSE),
-                                                       mEncoder(nsnull)
+                                                       mDidWrite(false),
+                                                       mEncoder(nullptr)
 {
 }
 
@@ -362,7 +284,7 @@ nsJSONWriter::SetCharset(const char* aCharset)
     rv = ccm->GetUnicodeEncoder(aCharset, getter_AddRefs(mEncoder));
     NS_ENSURE_SUCCESS(rv, rv);
     rv = mEncoder->SetOutputErrorBehavior(nsIUnicodeEncoder::kOnError_Signal,
-                                          nsnull, nsnull);
+                                          nullptr, '\0');
     NS_ENSURE_SUCCESS(rv, rv);
   }
 
@@ -370,7 +292,7 @@ nsJSONWriter::SetCharset(const char* aCharset)
 }
 
 nsresult
-nsJSONWriter::Write(const PRUnichar *aBuffer, PRUint32 aLength)
+nsJSONWriter::Write(const PRUnichar *aBuffer, uint32_t aLength)
 {
   if (mStream) {
     return WriteToStream(mStream, mEncoder, aBuffer, aLength);
@@ -380,7 +302,7 @@ nsJSONWriter::Write(const PRUnichar *aBuffer, PRUint32 aLength)
     mBuffer = new PRUnichar[JSON_STREAM_BUFSIZE];
     if (!mBuffer)
       return NS_ERROR_OUT_OF_MEMORY;
-    mDidWrite = PR_TRUE;
+    mDidWrite = true;
   }
 
   if (JSON_STREAM_BUFSIZE <= aLength + mBufferCount) {
@@ -399,7 +321,7 @@ nsJSONWriter::Write(const PRUnichar *aBuffer, PRUint32 aLength)
   return NS_OK;
 }
 
-PRBool nsJSONWriter::DidWrite()
+bool nsJSONWriter::DidWrite()
 {
   return mDidWrite;
 }
@@ -414,14 +336,14 @@ nsresult
 nsJSONWriter::WriteToStream(nsIOutputStream *aStream,
                             nsIUnicodeEncoder *encoder,
                             const PRUnichar *aBuffer,
-                            PRUint32 aLength)
+                            uint32_t aLength)
 {
   nsresult rv;
-  PRInt32 srcLength = aLength;
-  PRUint32 bytesWritten;
+  int32_t srcLength = aLength;
+  uint32_t bytesWritten;
 
   // The bytes written to the stream might differ from the PRUnichar size
-  PRInt32 aDestLength;
+  int32_t aDestLength;
   rv = encoder->GetMaxLength(aBuffer, srcLength, &aDestLength);
   NS_ENSURE_SUCCESS(rv, rv);
 
@@ -435,33 +357,34 @@ nsJSONWriter::WriteToStream(nsIOutputStream *aStream,
     rv = aStream->Write(destBuf, aDestLength, &bytesWritten);
 
   NS_Free(destBuf);
-  mDidWrite = PR_TRUE;
+  mDidWrite = true;
 
   return rv;
 }
 
 NS_IMETHODIMP
-nsJSON::Decode(const nsAString& json)
+nsJSON::Decode(const nsAString& json, JSContext* cx, JS::Value* aRetval)
 {
   nsresult rv = WarnDeprecatedMethod(DecodeWarning);
   if (NS_FAILED(rv))
     return rv;
 
   const PRUnichar *data;
-  PRUint32 len = NS_StringGetData(json, &data);
+  uint32_t len = NS_StringGetData(json, &data);
   nsCOMPtr<nsIInputStream> stream;
   rv = NS_NewByteInputStream(getter_AddRefs(stream),
                              reinterpret_cast<const char*>(data),
                              len * sizeof(PRUnichar),
                              NS_ASSIGNMENT_DEPEND);
   NS_ENSURE_SUCCESS(rv, rv);
-  return DecodeInternal(stream, len, PR_FALSE);
+  return DecodeInternal(cx, stream, len, false, aRetval);
 }
 
 NS_IMETHODIMP
-nsJSON::DecodeFromStream(nsIInputStream *aStream, PRInt32 aContentLength)
+nsJSON::DecodeFromStream(nsIInputStream *aStream, int32_t aContentLength,
+                         JSContext* cx, JS::Value* aRetval)
 {
-  return DecodeInternal(aStream, aContentLength, PR_TRUE);
+  return DecodeInternal(cx, aStream, aContentLength, true, aRetval);
 }
 
 NS_IMETHODIMP
@@ -469,8 +392,8 @@ nsJSON::DecodeToJSVal(const nsAString &str, JSContext *cx, jsval *result)
 {
   JSAutoRequest ar(cx);
 
-  if (!JS_ParseJSON(cx, (jschar*)PromiseFlatString(str).get(),
-                    (uint32)str.Length(), result)) {
+  if (!JS_ParseJSON(cx, static_cast<const jschar*>(PromiseFlatString(str).get()),
+                    str.Length(), result)) {
     return NS_ERROR_UNEXPECTED;
   }
 
@@ -478,28 +401,13 @@ nsJSON::DecodeToJSVal(const nsAString &str, JSContext *cx, jsval *result)
 }
 
 nsresult
-nsJSON::DecodeInternal(nsIInputStream *aStream,
-                       PRInt32 aContentLength,
-                       PRBool aNeedsConverter,
+nsJSON::DecodeInternal(JSContext* cx,
+                       nsIInputStream *aStream,
+                       int32_t aContentLength,
+                       bool aNeedsConverter,
+                       JS::Value* aRetval,
                        DecodingMode mode /* = STRICT */)
 {
-  nsresult rv;
-  nsIXPConnect *xpc = nsContentUtils::XPConnect();
-  if (!xpc)
-    return NS_ERROR_FAILURE;
-
-  nsAXPCNativeCallContext *cc = nsnull;
-  rv = xpc->GetCurrentNativeCallContext(&cc);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  jsval *retvalPtr;
-  rv = cc->GetRetValPtr(&retvalPtr);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  JSContext *cx = nsnull;
-  rv = cc->GetJSContext(&cx);
-  NS_ENSURE_SUCCESS(rv, rv);
-
   JSAutoRequest ar(cx);
 
   // Consume the stream
@@ -510,19 +418,17 @@ nsJSON::DecodeInternal(nsIInputStream *aStream,
       return NS_ERROR_OUT_OF_MEMORY;
   }
 
-  rv = NS_NewInputStreamChannel(getter_AddRefs(jsonChannel), mURI, aStream,
-                                NS_LITERAL_CSTRING("application/json"));
+  nsresult rv =
+    NS_NewInputStreamChannel(getter_AddRefs(jsonChannel), mURI, aStream,
+                             NS_LITERAL_CSTRING("application/json"));
   if (!jsonChannel || NS_FAILED(rv))
     return NS_ERROR_FAILURE;
 
-  nsRefPtr<nsJSONListener>
-    jsonListener(new nsJSONListener(cx, retvalPtr, aNeedsConverter, mode));
-
-  if (!jsonListener)
-    return NS_ERROR_OUT_OF_MEMORY;
+  nsRefPtr<nsJSONListener> jsonListener =
+    new nsJSONListener(cx, aRetval, aNeedsConverter, mode);
 
   //XXX this stream pattern should be consolidated in netwerk
-  rv = jsonListener->OnStartRequest(jsonChannel, nsnull);
+  rv = jsonListener->OnStartRequest(jsonChannel, nullptr);
   if (NS_FAILED(rv)) {
     jsonChannel->Cancel(rv);
     return rv;
@@ -530,9 +436,9 @@ nsJSON::DecodeInternal(nsIInputStream *aStream,
 
   nsresult status;
   jsonChannel->GetStatus(&status);
-  PRUint32 offset = 0;
+  uint64_t offset = 0;
   while (NS_SUCCEEDED(status)) {
-    PRUint32 available;
+    uint64_t available;
     rv = aStream->Available(&available);
     if (rv == NS_BASE_STREAM_CLOSED) {
       rv = NS_OK;
@@ -545,8 +451,13 @@ nsJSON::DecodeInternal(nsIInputStream *aStream,
     if (!available)
       break; // blocking input stream has none available when done
 
-    rv = jsonListener->OnDataAvailable(jsonChannel, nsnull,
-                                       aStream, offset, available);
+    if (available > UINT32_MAX)
+      available = UINT32_MAX;
+
+    rv = jsonListener->OnDataAvailable(jsonChannel, nullptr,
+                                       aStream,
+                                       offset,
+                                       (uint32_t)available);
     if (NS_FAILED(rv)) {
       jsonChannel->Cancel(rv);
       break;
@@ -557,10 +468,7 @@ nsJSON::DecodeInternal(nsIInputStream *aStream,
   }
   NS_ENSURE_SUCCESS(rv, rv);
 
-  rv = jsonListener->OnStopRequest(jsonChannel, nsnull, status);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  rv = cc->SetReturnValueWasSet(PR_TRUE);
+  rv = jsonListener->OnStopRequest(jsonChannel, nullptr, status);
   NS_ENSURE_SUCCESS(rv, rv);
 
   return NS_OK;
@@ -568,23 +476,24 @@ nsJSON::DecodeInternal(nsIInputStream *aStream,
 
 
 NS_IMETHODIMP
-nsJSON::LegacyDecode(const nsAString& json)
+nsJSON::LegacyDecode(const nsAString& json, JSContext* cx, JS::Value* aRetval)
 {
   const PRUnichar *data;
-  PRUint32 len = NS_StringGetData(json, &data);
+  uint32_t len = NS_StringGetData(json, &data);
   nsCOMPtr<nsIInputStream> stream;
   nsresult rv = NS_NewByteInputStream(getter_AddRefs(stream),
                                       (const char*) data,
                                       len * sizeof(PRUnichar),
                                       NS_ASSIGNMENT_DEPEND);
   NS_ENSURE_SUCCESS(rv, rv);
-  return DecodeInternal(stream, len, PR_FALSE, LEGACY);
+  return DecodeInternal(cx, stream, len, false, aRetval, LEGACY);
 }
 
 NS_IMETHODIMP
-nsJSON::LegacyDecodeFromStream(nsIInputStream *aStream, PRInt32 aContentLength)
+nsJSON::LegacyDecodeFromStream(nsIInputStream *aStream, int32_t aContentLength,
+                               JSContext* cx, JS::Value* aRetval)
 {
-  return DecodeInternal(aStream, aContentLength, PR_TRUE, LEGACY);
+  return DecodeInternal(cx, aStream, aContentLength, true, aRetval, LEGACY);
 }
 
 NS_IMETHODIMP
@@ -592,12 +501,16 @@ nsJSON::LegacyDecodeToJSVal(const nsAString &str, JSContext *cx, jsval *result)
 {
   JSAutoRequest ar(cx);
 
-  if (!js::ParseJSONWithReviver(cx, (jschar*)PromiseFlatString(str).get(),
-                                (uint32)str.Length(), js::NullValue(),
-                                js::Valueify(result), LEGACY)) {
+  js::RootedValue reviver(cx, JS::NullValue()), value(cx);
+
+  JS::StableCharPtr chars(static_cast<const jschar*>(PromiseFlatString(str).get()),
+                          str.Length());
+  if (!js::ParseJSONWithReviver(cx, chars, str.Length(), reviver,
+                                &value, LEGACY)) {
     return NS_ERROR_UNEXPECTED;
   }
 
+  *result = value;
   return NS_OK;
 }
 
@@ -615,7 +528,7 @@ NS_NewJSON(nsISupports* aOuter, REFNSIID aIID, void** aResult)
 }
 
 nsJSONListener::nsJSONListener(JSContext *cx, jsval *rootVal,
-                               PRBool needsConverter,
+                               bool needsConverter,
                                DecodingMode mode /* = STRICT */)
   : mNeedsConverter(needsConverter), 
     mCx(cx),
@@ -641,7 +554,7 @@ NS_IMETHODIMP
 nsJSONListener::OnStartRequest(nsIRequest *aRequest, nsISupports *aContext)
 {
   mSniffBuffer.Truncate();
-  mDecoder = nsnull;
+  mDecoder = nullptr;
 
   return NS_OK;
 }
@@ -655,15 +568,20 @@ nsJSONListener::OnStopRequest(nsIRequest *aRequest, nsISupports *aContext,
   // This can happen with short UTF-8 messages (<4 bytes)
   if (!mSniffBuffer.IsEmpty()) {
     // Just consume mSniffBuffer
-    rv = ProcessBytes(nsnull, 0);
+    rv = ProcessBytes(nullptr, 0);
     NS_ENSURE_SUCCESS(rv, rv);
   }
 
-  const jschar* chars = reinterpret_cast<const jschar*>(mBufferedChars.Elements());
+  js::RootedValue reviver(mCx, JS::NullValue()), value(mCx);
+
+  JS::StableCharPtr chars(reinterpret_cast<const jschar*>(mBufferedChars.Elements()),
+                          mBufferedChars.Length());
   JSBool ok = js::ParseJSONWithReviver(mCx, chars,
-                                       (uint32) mBufferedChars.Length(),
-                                       js::NullValue(), js::Valueify(mRootVal),
+                                       (uint32_t) mBufferedChars.Length(),
+                                       reviver, &value,
                                        mDecodingMode);
+
+  *mRootVal = value;
   mBufferedChars.TruncateLength(0);
   return ok ? NS_OK : NS_ERROR_FAILURE;
 }
@@ -671,14 +589,12 @@ nsJSONListener::OnStopRequest(nsIRequest *aRequest, nsISupports *aContext,
 NS_IMETHODIMP
 nsJSONListener::OnDataAvailable(nsIRequest *aRequest, nsISupports *aContext,
                                 nsIInputStream *aStream,
-                                PRUint32 aOffset, PRUint32 aLength)
+                                uint64_t aOffset, uint32_t aLength)
 {
-  PRUint32 contentLength;
-  aStream->Available(&contentLength);
   nsresult rv = NS_OK;
 
   if (mNeedsConverter && mSniffBuffer.Length() < 4) {
-    PRUint32 readCount = (aLength < 4) ? aLength : 4;
+    uint32_t readCount = (aLength < 4) ? aLength : 4;
     rv = NS_ConsumeStream(aStream, readCount, mSniffBuffer);
     NS_ENSURE_SUCCESS(rv, rv);
 
@@ -691,7 +607,7 @@ nsJSONListener::OnDataAvailable(nsIRequest *aRequest, nsISupports *aContext,
   while (bytesRemaining) {
     unsigned int bytesRead;
     rv = aStream->Read(buffer,
-                       NS_MIN((unsigned long)sizeof(buffer), bytesRemaining),
+                       std::min((unsigned long)sizeof(buffer), bytesRemaining),
                        &bytesRead);
     NS_ENSURE_SUCCESS(rv, rv);
     rv = ProcessBytes(buffer, bytesRead);
@@ -703,11 +619,11 @@ nsJSONListener::OnDataAvailable(nsIRequest *aRequest, nsISupports *aContext,
 }
 
 nsresult
-nsJSONListener::ProcessBytes(const char* aBuffer, PRUint32 aByteLength)
+nsJSONListener::ProcessBytes(const char* aBuffer, uint32_t aByteLength)
 {
   nsresult rv;
   // Check for BOM, or sniff charset
-  nsCAutoString charset;
+  nsAutoCString charset;
   if (mNeedsConverter && !mDecoder) {
     if (!nsContentUtils::CheckForBOM((const unsigned char*) mSniffBuffer.get(),
                                       mSniffBuffer.Length(), charset)) {
@@ -752,7 +668,7 @@ nsJSONListener::ProcessBytes(const char* aBuffer, PRUint32 aByteLength)
   if (mNeedsConverter) {
     rv = ConsumeConverted(aBuffer, aByteLength);
   } else {
-    PRUint32 unichars = aByteLength / sizeof(PRUnichar);
+    uint32_t unichars = aByteLength / sizeof(PRUnichar);
     rv = Consume((PRUnichar *) aBuffer, unichars);
   }
 
@@ -760,17 +676,17 @@ nsJSONListener::ProcessBytes(const char* aBuffer, PRUint32 aByteLength)
 }
 
 nsresult
-nsJSONListener::ConsumeConverted(const char* aBuffer, PRUint32 aByteLength)
+nsJSONListener::ConsumeConverted(const char* aBuffer, uint32_t aByteLength)
 {
   nsresult rv;
-  PRInt32 unicharLength = 0;
-  PRInt32 srcLen = aByteLength;
+  int32_t unicharLength = 0;
+  int32_t srcLen = aByteLength;
 
   rv = mDecoder->GetMaxLength(aBuffer, srcLen, &unicharLength);
   NS_ENSURE_SUCCESS(rv, rv);
 
   PRUnichar* endelems = mBufferedChars.AppendElements(unicharLength);
-  PRInt32 preLength = unicharLength;
+  int32_t preLength = unicharLength;
   rv = mDecoder->Convert(aBuffer, &srcLen, endelems, &unicharLength);
   if (NS_FAILED(rv))
     return rv;
@@ -781,7 +697,7 @@ nsJSONListener::ConsumeConverted(const char* aBuffer, PRUint32 aByteLength)
 }
 
 nsresult
-nsJSONListener::Consume(const PRUnichar* aBuffer, PRUint32 aByteLength)
+nsJSONListener::Consume(const PRUnichar* aBuffer, uint32_t aByteLength)
 {
   if (!mBufferedChars.AppendElements(aBuffer, aByteLength))
     return NS_ERROR_FAILURE;
