@@ -1,41 +1,9 @@
 /* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
  * vim: set ts=4 sw=4 et tw=99:
  *
- * ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is Mozilla SpiderMonkey JavaScript 1.9 code, released
- * May 28, 2008.
- *
- * The Initial Developer of the Original Code is
- *   Brendan Eich <brendan@mozilla.org>
- *
- * Contributor(s):
- *   David Anderson <danderson@mozilla.com>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either of the GNU General Public License Version 2 or later (the "GPL"),
- * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+ * This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #ifndef js_typedarray_ic_h___
 #define js_typedarray_ic_h___
@@ -43,13 +11,15 @@
 #include "jscntxt.h"
 #include "jstypedarray.h"
 
+#include "vm/NumericConversions.h"
+
 #include "jsnuminlines.h"
 #include "jstypedarrayinlines.h"
 
 namespace js {
 namespace mjit {
 
-#ifdef JS_POLYIC_TYPED_ARRAY
+#ifdef JS_METHODJIT_TYPED_ARRAY
 
 typedef JSC::MacroAssembler::RegisterID RegisterID;
 typedef JSC::MacroAssembler::FPRegisterID FPRegisterID;
@@ -57,62 +27,6 @@ typedef JSC::MacroAssembler::Jump Jump;
 typedef JSC::MacroAssembler::Imm32 Imm32;
 typedef JSC::MacroAssembler::ImmDouble ImmDouble;
 
-template <typename T>
-static void
-LoadFromTypedArray(Assembler &masm, JSObject *tarray, T address,
-                   RegisterID typeReg, RegisterID dataReg)
-{
-    switch (TypedArray::getType(tarray)) {
-      case js::TypedArray::TYPE_INT8:
-        masm.load8SignExtend(address, dataReg);
-        masm.move(ImmType(JSVAL_TYPE_INT32), typeReg);
-        break;
-      case js::TypedArray::TYPE_UINT8:
-      case js::TypedArray::TYPE_UINT8_CLAMPED:
-        masm.load8ZeroExtend(address, dataReg);
-        masm.move(ImmType(JSVAL_TYPE_INT32), typeReg);
-        break;
-      case js::TypedArray::TYPE_INT16:
-        masm.load16SignExtend(address, dataReg);
-        masm.move(ImmType(JSVAL_TYPE_INT32), typeReg);
-        break;
-      case js::TypedArray::TYPE_UINT16:
-        masm.load16(address, dataReg);
-        masm.move(ImmType(JSVAL_TYPE_INT32), typeReg);
-        break;
-      case js::TypedArray::TYPE_INT32:
-        masm.load32(address, dataReg);
-        masm.move(ImmType(JSVAL_TYPE_INT32), typeReg);
-        break;
-      case js::TypedArray::TYPE_UINT32:
-      {
-        masm.load32(address, dataReg);
-        masm.move(ImmType(JSVAL_TYPE_INT32), typeReg);
-        Jump safeInt = masm.branch32(Assembler::Below, dataReg, Imm32(0x80000000));
-        masm.convertUInt32ToDouble(dataReg, FPRegisters::First);
-        masm.breakDouble(FPRegisters::First, typeReg, dataReg);
-        safeInt.linkTo(masm.label(), &masm);
-        break;
-      }
-      case js::TypedArray::TYPE_FLOAT32:
-      case js::TypedArray::TYPE_FLOAT64:
-      {
-        if (TypedArray::getType(tarray) == js::TypedArray::TYPE_FLOAT32)
-            masm.loadFloat(address, FPRegisters::First);
-        else
-            masm.loadDouble(address, FPRegisters::First);
-        // Make sure NaN gets canonicalized.
-        Jump notNaN = masm.branchDouble(Assembler::DoubleEqual,
-                                        FPRegisters::First,
-                                        FPRegisters::First);
-        masm.loadStaticDouble(&js_NaN, FPRegisters::First, dataReg);
-        notNaN.linkTo(masm.label(), &masm);
-        masm.breakDouble(FPRegisters::First, typeReg, dataReg);
-        break;
-      }
-    }
-}
-
 static inline bool
 ConstantFoldForFloatArray(JSContext *cx, ValueRemat *vr)
 {
@@ -137,10 +51,10 @@ ConstantFoldForFloatArray(JSContext *cx, ValueRemat *vr)
     if (vr->knownType() == JSVAL_TYPE_DOUBLE)
         return true;
 
-    jsdouble d = 0;
+    double d = 0;
     Value v = vr->value();
     if (v.isString()) {
-        if (!StringToNumberType<jsdouble>(cx, v.toString(), &d))
+        if (!StringToNumberType<double>(cx, v.toString(), &d))
             return false;
     } else if (v.isBoolean()) {
         d = v.toBoolean() ? 1 : 0;
@@ -153,16 +67,6 @@ ConstantFoldForFloatArray(JSContext *cx, ValueRemat *vr)
     return true;
 }
 
-static inline int32
-ClampIntForUint8Array(int32 x)
-{
-    if (x < 0)
-        return 0;
-    if (x > 255)
-        return 255;
-    return x;
-}
-
 static inline bool
 ConstantFoldForIntArray(JSContext *cx, JSObject *tarray, ValueRemat *vr)
 {
@@ -190,14 +94,14 @@ ConstantFoldForIntArray(JSContext *cx, JSObject *tarray, ValueRemat *vr)
         v.setNumber(d);
     }
 
-    int32 i32 = 0;
+    int32_t i32 = 0;
     if (v.isDouble()) {
-        i32 = (TypedArray::getType(tarray) == js::TypedArray::TYPE_UINT8_CLAMPED)
-              ? js_TypedArray_uint8_clamp_double(v.toDouble())
-              : js_DoubleToECMAInt32(v.toDouble());
+        i32 = (TypedArray::type(tarray) == js::TypedArray::TYPE_UINT8_CLAMPED)
+              ? ClampDoubleToUint8(v.toDouble())
+              : ToInt32(v.toDouble());
     } else if (v.isInt32()) {
         i32 = v.toInt32();
-        if (TypedArray::getType(tarray) == js::TypedArray::TYPE_UINT8_CLAMPED)
+        if (TypedArray::type(tarray) == js::TypedArray::TYPE_UINT8_CLAMPED)
             i32 = ClampIntForUint8Array(i32);
     } else if (v.isBoolean()) {
         i32 = v.toBoolean() ? 1 : 0;
@@ -210,39 +114,6 @@ ConstantFoldForIntArray(JSContext *cx, JSObject *tarray, ValueRemat *vr)
     return true;
 }
 
-template <typename S, typename T>
-static void
-StoreToIntArray(Assembler &masm, JSObject *tarray, S src, T address)
-{
-    switch (TypedArray::getType(tarray)) {
-      case js::TypedArray::TYPE_INT8:
-      case js::TypedArray::TYPE_UINT8:
-      case js::TypedArray::TYPE_UINT8_CLAMPED:
-        masm.store8(src, address);
-        break;
-      case js::TypedArray::TYPE_INT16:
-      case js::TypedArray::TYPE_UINT16:
-        masm.store16(src, address);
-        break;
-      case js::TypedArray::TYPE_INT32:
-      case js::TypedArray::TYPE_UINT32:
-        masm.store32(src, address);
-        break;
-      default:
-        JS_NOT_REACHED("unknown int array type");
-    }
-}
-
-template <typename S, typename T>
-static void
-StoreToFloatArray(Assembler &masm, JSObject *tarray, S src, T address)
-{
-    if (TypedArray::getType(tarray) == js::TypedArray::TYPE_FLOAT32)
-        masm.storeFloat(src, address);
-    else
-        masm.storeDouble(src, address);
-}
-
 // Generate code that will ensure a dynamically typed value, pinned in |vr|,
 // can be stored in an integer typed array. If any sort of conversion is
 // required, |dataReg| will be clobbered by a new value. |saveMask| is
@@ -250,7 +121,7 @@ StoreToFloatArray(Assembler &masm, JSObject *tarray, S src, T address)
 // across any conversion process.
 static void
 GenConversionForIntArray(Assembler &masm, JSObject *tarray, const ValueRemat &vr,
-                         uint32 saveMask)
+                         uint32_t saveMask)
 {
     if (vr.isConstant()) {
         // Constants are always folded to ints up-front.
@@ -276,11 +147,11 @@ GenConversionForIntArray(Assembler &masm, JSObject *tarray, const ValueRemat &vr
         masm.storeArg(0, masm.vmFrameOffset(offsetof(VMFrame, cx)));
         masm.storeArgAddr(1, masm.addressOfExtra(vp));
 
-        typedef int32 (JS_FASTCALL *Int32CxVp)(JSContext *, Value *);
+        typedef int32_t (JS_FASTCALL *Int32CxVp)(JSContext *, Value *);
         Int32CxVp stub;
-        if (TypedArray::getType(tarray) == js::TypedArray::TYPE_UINT8_CLAMPED)
+        if (TypedArray::type(tarray) == js::TypedArray::TYPE_UINT8_CLAMPED)
             stub = stubs::ConvertToTypedInt<true>;
-        else 
+        else
             stub = stubs::ConvertToTypedInt<false>;
         masm.callWithABI(JS_FUNC_TO_DATA_PTR(void *, stub), false);
         if (vr.dataReg() != Registers::ReturnReg)
@@ -294,26 +165,8 @@ GenConversionForIntArray(Assembler &masm, JSObject *tarray, const ValueRemat &vr
     }
 
     // Performing clamping, if needed.
-    if (TypedArray::getType(tarray) == js::TypedArray::TYPE_UINT8_CLAMPED) {
-        //     cmp dr, 0
-        //     jge _min
-        //     mov dr, 0
-        //     jump _done
-        // _min:
-        //     cmp dr, 255
-        //     jle _done
-        //     mov dr, 255
-        // _done:
-        //
-        Jump j = masm.branch32(Assembler::GreaterThanOrEqual, vr.dataReg(), Imm32(0));
-        masm.move(Imm32(0), vr.dataReg());
-        Jump done = masm.jump();
-        j.linkTo(masm.label(), &masm);
-        j = masm.branch32(Assembler::LessThanOrEqual, vr.dataReg(), Imm32(255));
-        masm.move(Imm32(255), vr.dataReg());
-        j.linkTo(masm.label(), &masm);
-        done.linkTo(masm.label(), &masm);
-    }
+    if (TypedArray::type(tarray) == js::TypedArray::TYPE_UINT8_CLAMPED)
+        masm.clampInt32ToUint8(vr.dataReg());
 }
 
 // Generate code that will ensure a dynamically typed value, pinned in |vr|,
@@ -321,11 +174,10 @@ GenConversionForIntArray(Assembler &masm, JSObject *tarray, const ValueRemat &vr
 // |dataReg| (and volatile registers) are preserved across any conversion
 // process.
 //
-// Constants are left untouched. Any other value is placed into
-// FPRegisters::First.
+// Constants are left untouched. Any other value is placed into destReg.
 static void
 GenConversionForFloatArray(Assembler &masm, JSObject *tarray, const ValueRemat &vr,
-                           FPRegisterID destReg, uint32 saveMask)
+                           FPRegisterID destReg, uint32_t saveMask)
 {
     if (vr.isConstant()) {
         // Constants are always folded to doubles up-front.
@@ -351,7 +203,7 @@ GenConversionForFloatArray(Assembler &masm, JSObject *tarray, const ValueRemat &
         }
     }
 
-    // Generate a generic conversion call, if not known to be int32 or double.
+    // Generate a generic conversion call, if not known to be int32_t or double.
     if (!vr.isTypeKnown() ||
         (vr.knownType() != JSVAL_TYPE_INT32 &&
          vr.knownType() != JSVAL_TYPE_DOUBLE)) {
@@ -389,18 +241,19 @@ GenConversionForFloatArray(Assembler &masm, JSObject *tarray, const ValueRemat &
     if (skip2.isSet())
         skip2.get().linkTo(masm.label(), &masm);
 
-    if (TypedArray::getType(tarray) == js::TypedArray::TYPE_FLOAT32)
+    if (TypedArray::type(tarray) == js::TypedArray::TYPE_FLOAT32)
         masm.convertDoubleToFloat(destReg, destReg);
 }
 
 template <typename T>
 static bool
 StoreToTypedArray(JSContext *cx, Assembler &masm, JSObject *tarray, T address,
-                  const ValueRemat &vrIn, uint32 saveMask)
+                  const ValueRemat &vrIn, uint32_t saveMask)
 {
     ValueRemat vr = vrIn;
 
-    switch (TypedArray::getType(tarray)) {
+    uint32_t type = TypedArray::type(tarray);
+    switch (type) {
       case js::TypedArray::TYPE_INT8:
       case js::TypedArray::TYPE_UINT8:
       case js::TypedArray::TYPE_UINT8_CLAMPED:
@@ -416,7 +269,7 @@ StoreToTypedArray(JSContext *cx, Assembler &masm, JSObject *tarray, T address,
         PreserveRegisters saveLHS(masm);
 
         // There are three tricky situations to handle:
-        //   (1) The RHS needs conversion. saveMask will be stomped, and 
+        //   (1) The RHS needs conversion. saveMask will be stomped, and
         //       the RHS may need to be stomped.
         //   (2) The RHS may need to be clamped, which clobbers it.
         //   (3) The RHS may need to be in a single-byte register.
@@ -429,11 +282,11 @@ StoreToTypedArray(JSContext *cx, Assembler &masm, JSObject *tarray, T address,
         // for the conversion call. This is because the object and key may be
         // in temporary registers, and we want to restore those without killing
         // the mutated RHS.
-        bool singleByte = (TypedArray::getType(tarray) == js::TypedArray::TYPE_INT8 ||
-                           TypedArray::getType(tarray) == js::TypedArray::TYPE_UINT8 ||
-                           TypedArray::getType(tarray) == js::TypedArray::TYPE_UINT8_CLAMPED);
+        bool singleByte = (type == js::TypedArray::TYPE_INT8 ||
+                           type == js::TypedArray::TYPE_UINT8 ||
+                           type == js::TypedArray::TYPE_UINT8_CLAMPED);
         bool mayNeedConversion = (!vr.isTypeKnown() || vr.knownType() != JSVAL_TYPE_INT32);
-        bool mayNeedClamping = !vr.isConstant() && (TypedArray::getType(tarray) == js::TypedArray::TYPE_UINT8_CLAMPED);
+        bool mayNeedClamping = !vr.isConstant() && (type == js::TypedArray::TYPE_UINT8_CLAMPED);
         bool needsSingleByteReg = singleByte &&
                                   !vr.isConstant() &&
                                   !(Registers::SingleByteRegs & Registers::maskReg(vr.dataReg()));
@@ -445,12 +298,12 @@ StoreToTypedArray(JSContext *cx, Assembler &masm, JSObject *tarray, T address,
             //   - won't clobber the key, object, or RHS type regs
             //   - is temporary, but
             //   - is not in saveMask, which contains live volatile registers.
-            uint32 allowMask = Registers::AvailRegs;
+            uint32_t allowMask = Registers::AvailRegs;
             if (singleByte)
                 allowMask &= Registers::SingleByteRegs;
 
             // Create a mask of registers we absolutely cannot clobber.
-            uint32 pinned = Assembler::maskAddress(address);
+            uint32_t pinned = Assembler::maskAddress(address);
             if (!vr.isTypeKnown())
                 pinned |= Registers::maskReg(vr.typeReg());
 
@@ -458,13 +311,13 @@ StoreToTypedArray(JSContext *cx, Assembler &masm, JSObject *tarray, T address,
 
             RegisterID newReg;
             if (!avail.empty()) {
-                newReg = avail.takeAnyReg();
+                newReg = avail.takeAnyReg().reg();
             } else {
                 // If no registers meet the ideal set, relax a constraint and spill.
                 avail = allowMask & ~pinned;
 
                 if (!avail.empty()) {
-                    newReg = avail.takeAnyReg();
+                    newReg = avail.takeAnyReg().reg();
                     saveRHS.preserve(Registers::maskReg(newReg));
                 } else {
                     // Oh no! *All* single byte registers are pinned. This
@@ -473,12 +326,12 @@ StoreToTypedArray(JSContext *cx, Assembler &masm, JSObject *tarray, T address,
 
                     // If |vr|'s registers are part of the address, swapping is
                     // going to cause problems during the store.
-                    uint32 vrRegs = Registers::mask2Regs(vr.dataReg(), vr.typeReg());
-                    uint32 lhsMask = vrRegs & Assembler::maskAddress(address);
+                    uint32_t vrRegs = Registers::mask2Regs(vr.dataReg(), vr.typeReg());
+                    uint32_t lhsMask = vrRegs & Assembler::maskAddress(address);
 
                     // We'll also need to save any of the registers which won't
                     // be restored via |lhsMask| above.
-                    uint32 rhsMask = vrRegs & ~lhsMask;
+                    uint32_t rhsMask = vrRegs & ~lhsMask;
 
                     // Push them, but get the order right. We'll pop LHS first.
                     saveRHS.preserve(rhsMask);
@@ -515,9 +368,9 @@ StoreToTypedArray(JSContext *cx, Assembler &masm, JSObject *tarray, T address,
         saveLHS.restore();
 
         if (vr.isConstant())
-            StoreToIntArray(masm, tarray, Imm32(vr.value().toInt32()), address);
+            masm.storeToTypedIntArray(type, Imm32(vr.value().toInt32()), address);
         else
-            StoreToIntArray(masm, tarray, vr.dataReg(), address);
+            masm.storeToTypedIntArray(type, vr.dataReg(), address);
 
         // Note that this will finish restoring the damage from the
         // earlier register swap.
@@ -526,21 +379,29 @@ StoreToTypedArray(JSContext *cx, Assembler &masm, JSObject *tarray, T address,
       }
 
       case js::TypedArray::TYPE_FLOAT32:
-      case js::TypedArray::TYPE_FLOAT64:
+      case js::TypedArray::TYPE_FLOAT64: {
+        /*
+         * Use a temporary for conversion. Inference is disabled, so no FP
+         * registers are live.
+         */
+        Registers regs(Registers::TempFPRegs);
+        FPRegisterID temp = regs.takeAnyReg().fpreg();
+
         if (!ConstantFoldForFloatArray(cx, &vr))
             return false;
-        GenConversionForFloatArray(masm, tarray, vr, FPRegisters::First, saveMask);
+        GenConversionForFloatArray(masm, tarray, vr, temp, saveMask);
         if (vr.isConstant())
-            StoreToFloatArray(masm, tarray, ImmDouble(vr.value().toDouble()), address);
+            masm.storeToTypedFloatArray(type, ImmDouble(vr.value().toDouble()), address);
         else
-            StoreToFloatArray(masm, tarray, FPRegisters::First, address);
+            masm.storeToTypedFloatArray(type, temp, address);
         break;
+      }
     }
 
     return true;
 }
 
-#endif // defined(JS_POLYIC) && (defined JS_CPU_X86 || defined JS_CPU_X64)
+#endif /* JS_METHODJIT_TYPED_ARRAY */
 
 } /* namespace mjit */
 } /* namespace js */
