@@ -1,42 +1,8 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 // vim:cindent:ts=2:et:sw=2:
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is Mozilla Communicator client code.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 1998
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   L. David Baron <dbaron@dbaron.org>
- *   Pierre Phaneuf <pp@ludusdesign.com>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either of the GNU General Public License Version 2 or later (the "GPL"),
- * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 /* representation of one line within a block frame, a CSS line box */
 
@@ -44,39 +10,43 @@
 #include "nsLineLayout.h"
 #include "prprf.h"
 #include "nsBlockFrame.h"
-#include "nsGkAtoms.h"
-#include "nsFrameManager.h"
+#include "nsIFrame.h"
+#include "nsPresArena.h"
 #ifdef IBMBIDI
 #include "nsBidiPresUtils.h"
 #endif
+#include "nsStyleStructInlines.h"
+#include "mozilla/Likely.h"
 
 #ifdef DEBUG
-static PRInt32 ctorCount;
-PRInt32 nsLineBox::GetCtorCount() { return ctorCount; }
+static int32_t ctorCount;
+int32_t nsLineBox::GetCtorCount() { return ctorCount; }
 #endif
 
-nsLineBox::nsLineBox(nsIFrame* aFrame, PRInt32 aCount, PRBool aIsBlock)
-  : mFirstChild(aFrame),
-    mBounds(0, 0, 0, 0),
-    mAscent(0),
-    mData(nsnull)
+#ifndef _MSC_VER
+// static nsLineBox constant; initialized in the header file.
+const uint32_t nsLineBox::kMinChildCountForHashtable;
+#endif
+
+nsLineBox::nsLineBox(nsIFrame* aFrame, int32_t aCount, bool aIsBlock)
+  : mFirstChild(aFrame)
+// NOTE: memory is already zeroed since we allocate with AllocateByObjectID.
 {
   MOZ_COUNT_CTOR(nsLineBox);
 #ifdef DEBUG
   ++ctorCount;
   NS_ASSERTION(!aIsBlock || aCount == 1, "Blocks must have exactly one child");
   nsIFrame* f = aFrame;
-  for (PRInt32 n = aCount; n > 0; f = f->GetNextSibling(), --n) {
-    NS_ASSERTION(aIsBlock == f->GetStyleDisplay()->IsBlockOutside(),
+  for (int32_t n = aCount; n > 0; f = f->GetNextSibling(), --n) {
+    NS_ASSERTION(aIsBlock == f->IsBlockOutside(),
                  "wrong kind of child frame");
   }
 #endif
 
-  mAllFlags = 0;
 #if NS_STYLE_CLEAR_NONE > 0
   mFlags.mBreakType = NS_STYLE_CLEAR_NONE;
 #endif
-  SetChildCount(aCount);
+  mChildCount = aCount;
   MarkDirty();
   mFlags.mBlock = aIsBlock;
 }
@@ -84,14 +54,86 @@ nsLineBox::nsLineBox(nsIFrame* aFrame, PRInt32 aCount, PRBool aIsBlock)
 nsLineBox::~nsLineBox()
 {
   MOZ_COUNT_DTOR(nsLineBox);
+  if (MOZ_UNLIKELY(mFlags.mHasHashedFrames)) {
+    delete mFrames;
+  }  
   Cleanup();
 }
 
 nsLineBox*
-NS_NewLineBox(nsIPresShell* aPresShell, nsIFrame* aFrame,
-              PRInt32 aCount, PRBool aIsBlock)
+NS_NewLineBox(nsIPresShell* aPresShell, nsIFrame* aFrame, bool aIsBlock)
 {
-  return new (aPresShell)nsLineBox(aFrame, aCount, aIsBlock);
+  return new (aPresShell) nsLineBox(aFrame, 1, aIsBlock);
+}
+
+nsLineBox*
+NS_NewLineBox(nsIPresShell* aPresShell, nsLineBox* aFromLine,
+              nsIFrame* aFrame, int32_t aCount)
+{
+  nsLineBox* newLine = new (aPresShell) nsLineBox(aFrame, aCount, false);
+  newLine->NoteFramesMovedFrom(aFromLine);
+  return newLine;
+}
+
+void
+nsLineBox::StealHashTableFrom(nsLineBox* aFromLine, uint32_t aFromLineNewCount)
+{
+  MOZ_ASSERT(!mFlags.mHasHashedFrames);
+  MOZ_ASSERT(GetChildCount() >= int32_t(aFromLineNewCount));
+  mFrames = aFromLine->mFrames;
+  mFlags.mHasHashedFrames = 1;
+  aFromLine->mFlags.mHasHashedFrames = 0;
+  aFromLine->mChildCount = aFromLineNewCount;
+  // remove aFromLine's frames that aren't on this line
+  nsIFrame* f = aFromLine->mFirstChild;
+  for (uint32_t i = 0; i < aFromLineNewCount; f = f->GetNextSibling(), ++i) {
+    mFrames->RemoveEntry(f);
+  }
+}
+
+void
+nsLineBox::NoteFramesMovedFrom(nsLineBox* aFromLine)
+{
+  uint32_t fromCount = aFromLine->GetChildCount();
+  uint32_t toCount = GetChildCount();
+  MOZ_ASSERT(toCount <= fromCount, "moved more frames than aFromLine has");
+  uint32_t fromNewCount = fromCount - toCount;
+  if (MOZ_LIKELY(!aFromLine->mFlags.mHasHashedFrames)) {
+    aFromLine->mChildCount = fromNewCount;
+    MOZ_ASSERT(toCount < kMinChildCountForHashtable);
+  } else if (fromNewCount < kMinChildCountForHashtable) {
+    // aFromLine has a hash table but will not have it after moving the frames
+    // so this line can steal the hash table if it needs it.
+    if (toCount >= kMinChildCountForHashtable) {
+      StealHashTableFrom(aFromLine, fromNewCount);
+    } else {
+      delete aFromLine->mFrames;
+      aFromLine->mFlags.mHasHashedFrames = 0;
+      aFromLine->mChildCount = fromNewCount;
+    }
+  } else {
+    // aFromLine still needs a hash table.
+    if (toCount < kMinChildCountForHashtable) {
+      // remove the moved frames from it
+      nsIFrame* f = mFirstChild;
+      for (uint32_t i = 0; i < toCount; f = f->GetNextSibling(), ++i) {
+        aFromLine->mFrames->RemoveEntry(f);
+      }
+    } else if (toCount <= fromNewCount) {
+      // This line needs a hash table, allocate a hash table for it since that
+      // means fewer hash ops.
+      nsIFrame* f = mFirstChild;
+      for (uint32_t i = 0; i < toCount; f = f->GetNextSibling(), ++i) {
+        aFromLine->mFrames->RemoveEntry(f); // toCount RemoveEntry
+      }
+      SwitchToHashtable(); // toCount PutEntry
+    } else {
+      // This line needs a hash table, but it's fewer hash ops to steal
+      // aFromLine's hash table and allocate a new hash table for that line.
+      StealHashTableFrom(aFromLine, fromNewCount); // fromNewCount RemoveEntry
+      aFromLine->SwitchToHashtable(); // fromNewCount PutEntry
+    }
+  }
 }
 
 // Overloaded new operator. Uses an arena (which comes from the presShell)
@@ -99,24 +141,14 @@ NS_NewLineBox(nsIPresShell* aPresShell, nsIFrame* aFrame,
 void*
 nsLineBox::operator new(size_t sz, nsIPresShell* aPresShell) CPP_THROW_NEW
 {
-  return aPresShell->AllocateMisc(sz);
-}
-
-// Overloaded delete operator. Doesn't actually free the memory, because we
-// use an arena
-void
-nsLineBox::operator delete(void* aPtr, size_t sz)
-{
+  return aPresShell->AllocateByObjectID(nsPresArena::nsLineBox_id, sz);
 }
 
 void
 nsLineBox::Destroy(nsIPresShell* aPresShell)
 {
-  // Destroy the object. This won't actually free the memory, though
-  delete this;
-
-  // Have the pres shell recycle the memory
-  aPresShell->FreeMisc(sizeof(*this), (void*)this);
+  this->nsLineBox::~nsLineBox();
+  aPresShell->FreeByObjectID(nsPresArena::nsLineBox_id, this);
 }
 
 void
@@ -129,13 +161,13 @@ nsLineBox::Cleanup()
     else {
       delete mInlineData;
     }
-    mData = nsnull;
+    mData = nullptr;
   }
 }
 
 #ifdef DEBUG
 static void
-ListFloats(FILE* out, PRInt32 aIndent, const nsFloatCacheList& aFloats)
+ListFloats(FILE* out, int32_t aIndent, const nsFloatCacheList& aFloats)
 {
   nsFloatCache* fc = aFloats.Head();
   while (fc) {
@@ -158,7 +190,7 @@ ListFloats(FILE* out, PRInt32 aIndent, const nsFloatCacheList& aFloats)
 
 #ifdef DEBUG
 const char *
-BreakTypeToString(PRUint8 aBreakType)
+BreakTypeToString(uint8_t aBreakType)
 {
   switch (aBreakType) {
   case NS_STYLE_CLEAR_NONE: return "nobr";
@@ -176,7 +208,7 @@ BreakTypeToString(PRUint8 aBreakType)
 }
 
 char*
-nsLineBox::StateToString(char* aBuf, PRInt32 aBufSize) const
+nsLineBox::StateToString(char* aBuf, int32_t aBufSize) const
 {
   PR_snprintf(aBuf, aBufSize, "%s,%s,%s,%s,%s,before:%s,after:%s[0x%x]",
               IsBlock() ? "block" : "inline",
@@ -191,11 +223,9 @@ nsLineBox::StateToString(char* aBuf, PRInt32 aBufSize) const
 }
 
 void
-nsLineBox::List(FILE* out, PRInt32 aIndent) const
+nsLineBox::List(FILE* out, int32_t aIndent, uint32_t aFlags) const
 {
-  PRInt32 i;
-
-  for (i = aIndent; --i >= 0; ) fputs("  ", out);
+  nsFrame::IndentBy(out, aIndent);
   char cbuf[100];
   fprintf(out, "line %p: count=%d state=%s ",
           static_cast<const void*>(this), GetChildCount(),
@@ -219,44 +249,39 @@ nsLineBox::List(FILE* out, PRInt32 aIndent) const
   fprintf(out, "<\n");
 
   nsIFrame* frame = mFirstChild;
-  PRInt32 n = GetChildCount();
+  int32_t n = GetChildCount();
   while (--n >= 0) {
-    frame->List(out, aIndent + 1);
+    frame->List(out, aIndent + 1, aFlags);
     frame = frame->GetNextSibling();
   }
 
-  for (i = aIndent; --i >= 0; ) fputs("  ", out);
   if (HasFloats()) {
+    nsFrame::IndentBy(out, aIndent);
     fputs("> floats <\n", out);
     ListFloats(out, aIndent + 1, mInlineData->mFloats);
-    for (i = aIndent; --i >= 0; ) fputs("  ", out);
   }
+  nsFrame::IndentBy(out, aIndent);
   fputs(">\n", out);
 }
 #endif
 
+#ifdef DEBUG
 nsIFrame*
 nsLineBox::LastChild() const
 {
   nsIFrame* frame = mFirstChild;
-  PRInt32 n = GetChildCount() - 1;
+  int32_t n = GetChildCount() - 1;
   while (--n >= 0) {
     frame = frame->GetNextSibling();
   }
   return frame;
 }
+#endif
 
-PRBool
-nsLineBox::IsLastChild(nsIFrame* aFrame) const
-{
-  nsIFrame* lastFrame = LastChild();
-  return aFrame == lastFrame;
-}
-
-PRInt32
+int32_t
 nsLineBox::IndexOf(nsIFrame* aFrame) const
 {
-  PRInt32 i, n = GetChildCount();
+  int32_t i, n = GetChildCount();
   nsIFrame* frame = mFirstChild;
   for (i = 0; i < n; i++) {
     if (frame == aFrame) {
@@ -267,28 +292,28 @@ nsLineBox::IndexOf(nsIFrame* aFrame) const
   return -1;
 }
 
-PRBool
+bool
 nsLineBox::IsEmpty() const
 {
   if (IsBlock())
     return mFirstChild->IsEmpty();
 
-  PRInt32 n;
+  int32_t n;
   nsIFrame *kid;
   for (n = GetChildCount(), kid = mFirstChild;
        n > 0;
        --n, kid = kid->GetNextSibling())
   {
     if (!kid->IsEmpty())
-      return PR_FALSE;
+      return false;
   }
   if (HasBullet()) {
-    return PR_FALSE;
+    return false;
   }
-  return PR_TRUE;
+  return true;
 }
 
-PRBool
+bool
 nsLineBox::CachedIsEmpty()
 {
   if (mFlags.mDirty) {
@@ -299,94 +324,93 @@ nsLineBox::CachedIsEmpty()
     return mFlags.mEmptyCacheState;
   }
 
-  PRBool result;
+  bool result;
   if (IsBlock()) {
     result = mFirstChild->CachedIsEmpty();
   } else {
-    PRInt32 n;
+    int32_t n;
     nsIFrame *kid;
-    result = PR_TRUE;
+    result = true;
     for (n = GetChildCount(), kid = mFirstChild;
          n > 0;
          --n, kid = kid->GetNextSibling())
       {
         if (!kid->CachedIsEmpty()) {
-          result = PR_FALSE;
+          result = false;
           break;
         }
       }
     if (HasBullet()) {
-      result = PR_FALSE;
+      result = false;
     }
   }
 
-  mFlags.mEmptyCacheValid = PR_TRUE;
+  mFlags.mEmptyCacheValid = true;
   mFlags.mEmptyCacheState = result;
   return result;
 }
 
 void
 nsLineBox::DeleteLineList(nsPresContext* aPresContext, nsLineList& aLines,
-                          nsIFrame* aDestructRoot)
+                          nsIFrame* aDestructRoot, nsFrameList* aFrames)
 {
-  if (! aLines.empty()) {
-    // Delete our child frames before doing anything else. In particular
-    // we do all of this before our base class releases it's hold on the
-    // view.
-#ifdef DEBUG
-    PRInt32 numFrames = 0;
-#endif
-    for (nsIFrame* child = aLines.front()->mFirstChild; child; ) {
-      nsIFrame* nextChild = child->GetNextSibling();
-      child->SetNextSibling(nsnull);
-      child->DestroyFrom((aDestructRoot) ? aDestructRoot : child);
-      child = nextChild;
-#ifdef DEBUG
-      numFrames++;
-#endif
+  nsIPresShell* shell = aPresContext->PresShell();
+
+  // Keep our line list and frame list up to date as we
+  // remove frames, in case something wants to traverse the
+  // frame tree while we're destroying.
+  while (!aLines.empty()) {
+    nsLineBox* line = aLines.front();
+    if (MOZ_UNLIKELY(line->mFlags.mHasHashedFrames)) {
+      line->SwitchToCounter();  // Avoid expensive has table removals.
+    }
+    while (line->GetChildCount() > 0) {
+      nsIFrame* child = aFrames->RemoveFirstChild();
+      MOZ_ASSERT(child == line->mFirstChild, "Lines out of sync");
+      line->mFirstChild = aFrames->FirstChild();
+      line->NoteFrameRemoved(child);
+      child->DestroyFrom(aDestructRoot);
     }
 
-    nsIPresShell *shell = aPresContext->PresShell();
-
-    do {
-      nsLineBox* line = aLines.front();
-#ifdef DEBUG
-      numFrames -= line->GetChildCount();
-#endif
-      aLines.pop_front();
-      line->Destroy(shell);
-    } while (! aLines.empty());
-#ifdef DEBUG
-    NS_ASSERTION(numFrames == 0, "number of frames deleted does not match");
-#endif
+    aLines.pop_front();
+    line->Destroy(shell);
   }
 }
 
-PRBool
+bool
 nsLineBox::RFindLineContaining(nsIFrame* aFrame,
                                const nsLineList::iterator& aBegin,
                                nsLineList::iterator& aEnd,
                                nsIFrame* aLastFrameBeforeEnd,
-                               PRInt32* aFrameIndexInLine)
+                               int32_t* aFrameIndexInLine)
 {
   NS_PRECONDITION(aFrame, "null ptr");
+
   nsIFrame* curFrame = aLastFrameBeforeEnd;
   while (aBegin != aEnd) {
     --aEnd;
-    NS_ASSERTION(aEnd->IsLastChild(curFrame), "Unexpected curFrame");
+    NS_ASSERTION(aEnd->LastChild() == curFrame, "Unexpected curFrame");
+    if (MOZ_UNLIKELY(aEnd->mFlags.mHasHashedFrames) &&
+        !aEnd->Contains(aFrame)) {
+      if (aEnd->mFirstChild) {
+        curFrame = aEnd->mFirstChild->GetPrevSibling();
+      }
+      continue;
+    }
     // i is the index of curFrame in aEnd
-    PRInt32 i = aEnd->GetChildCount() - 1;
+    int32_t i = aEnd->GetChildCount() - 1;
     while (i >= 0) {
       if (curFrame == aFrame) {
         *aFrameIndexInLine = i;
-        return PR_TRUE;
+        return true;
       }
       --i;
       curFrame = curFrame->GetPrevSibling();
     }
+    MOZ_ASSERT(!aEnd->mFlags.mHasHashedFrames, "Contains lied to us!");
   }
   *aFrameIndexInLine = -1;
-  return PR_FALSE;
+  return false;
 }
 
 nsCollapsingMargin
@@ -399,10 +423,10 @@ nsLineBox::GetCarriedOutBottomMargin() const
     : nsCollapsingMargin();
 }
 
-PRBool
+bool
 nsLineBox::SetCarriedOutBottomMargin(nsCollapsingMargin aValue)
 {
-  PRBool changed = PR_FALSE;
+  bool changed = false;
   if (IsBlock()) {
     if (!aValue.IsZero()) {
       if (!mBlockData) {
@@ -427,12 +451,12 @@ nsLineBox::MaybeFreeData()
     if (IsInline()) {
       if (mInlineData->mFloats.IsEmpty()) {
         delete mInlineData;
-        mInlineData = nsnull;
+        mInlineData = nullptr;
       }
     }
     else if (mBlockData->mCarriedOutBottomMargin.IsZero()) {
       delete mBlockData;
-      mBlockData = nsnull;
+      mBlockData = nullptr;
     }
   }
 }
@@ -442,7 +466,7 @@ nsFloatCache*
 nsLineBox::GetFirstFloat()
 {
   NS_ABORT_IF_FALSE(IsInline(), "block line can't have floats");
-  return mInlineData ? mInlineData->mFloats.Head() : nsnull;
+  return mInlineData ? mInlineData->mFloats.Head() : nullptr;
 }
 
 // XXX this might be too eager to free memory
@@ -472,7 +496,7 @@ nsLineBox::AppendFloats(nsFloatCacheFreeList& aFreeList)
   }
 }
 
-PRBool
+bool
 nsLineBox::RemoveFloat(nsIFrame* aFrame)
 {
   NS_ABORT_IF_FALSE(IsInline(), "block line can't have floats");
@@ -484,10 +508,10 @@ nsLineBox::RemoveFloat(nsIFrame* aFrame)
       mInlineData->mFloats.Remove(fc);
       delete fc;
       MaybeFreeData();
-      return PR_TRUE;
+      return true;
     }
   }
-  return PR_FALSE;
+  return false;
 }
 
 void
@@ -529,7 +553,7 @@ nsLineIterator::nsLineIterator()
   mLines = gDummyLines;
   mNumLines = 0;
   mIndex = 0;
-  mRightToLeft = PR_FALSE;
+  mRightToLeft = false;
 }
 
 nsLineIterator::~nsLineIterator()
@@ -546,12 +570,12 @@ nsLineIterator::DisposeLineIterator()
 }
 
 nsresult
-nsLineIterator::Init(nsLineList& aLines, PRBool aRightToLeft)
+nsLineIterator::Init(nsLineList& aLines, bool aRightToLeft)
 {
   mRightToLeft = aRightToLeft;
 
   // Count the lines
-  PRInt32 numLines = aLines.size();
+  int32_t numLines = aLines.size();
   if (0 == numLines) {
     // Use gDummyLines so that we don't need null pointer checks in
     // the accessor methods
@@ -578,31 +602,31 @@ nsLineIterator::Init(nsLineList& aLines, PRBool aRightToLeft)
   return NS_OK;
 }
 
-PRInt32
+int32_t
 nsLineIterator::GetNumLines()
 {
   return mNumLines;
 }
 
-PRBool
+bool
 nsLineIterator::GetDirection()
 {
   return mRightToLeft;
 }
 
 NS_IMETHODIMP
-nsLineIterator::GetLine(PRInt32 aLineNumber,
+nsLineIterator::GetLine(int32_t aLineNumber,
                         nsIFrame** aFirstFrameOnLine,
-                        PRInt32* aNumFramesOnLine,
+                        int32_t* aNumFramesOnLine,
                         nsRect& aLineBounds,
-                        PRUint32* aLineFlags)
+                        uint32_t* aLineFlags)
 {
   NS_ENSURE_ARG_POINTER(aFirstFrameOnLine);
   NS_ENSURE_ARG_POINTER(aNumFramesOnLine);
   NS_ENSURE_ARG_POINTER(aLineFlags);
 
   if ((aLineNumber < 0) || (aLineNumber >= mNumLines)) {
-    *aFirstFrameOnLine = nsnull;
+    *aFirstFrameOnLine = nullptr;
     *aNumFramesOnLine = 0;
     aLineBounds.SetRect(0, 0, 0, 0);
     return NS_OK;
@@ -612,7 +636,7 @@ nsLineIterator::GetLine(PRInt32 aLineNumber,
   *aNumFramesOnLine = line->GetChildCount();
   aLineBounds = line->mBounds;
 
-  PRUint32 flags = 0;
+  uint32_t flags = 0;
   if (line->IsBlock()) {
     flags |= NS_LINE_FLAG_IS_BLOCK;
   }
@@ -625,24 +649,25 @@ nsLineIterator::GetLine(PRInt32 aLineNumber,
   return NS_OK;
 }
 
-PRInt32
-nsLineIterator::FindLineContaining(nsIFrame* aFrame)
+int32_t
+nsLineIterator::FindLineContaining(nsIFrame* aFrame, int32_t aStartLine)
 {
-  nsLineBox* line = mLines[0];
-  PRInt32 lineNumber = 0;
+  NS_PRECONDITION(aStartLine <= mNumLines, "Bogus line numbers");
+  int32_t lineNumber = aStartLine;
   while (lineNumber != mNumLines) {
+    nsLineBox* line = mLines[lineNumber];
     if (line->Contains(aFrame)) {
       return lineNumber;
     }
-    line = mLines[++lineNumber];
+    ++lineNumber;
   }
   return -1;
 }
 
 #ifdef IBMBIDI
 NS_IMETHODIMP
-nsLineIterator::CheckLineOrder(PRInt32                  aLine,
-                               PRBool                   *aIsReordered,
+nsLineIterator::CheckLineOrder(int32_t                  aLine,
+                               bool                     *aIsReordered,
                                nsIFrame                 **aFirstVisual,
                                nsIFrame                 **aLastVisual)
 {
@@ -650,9 +675,9 @@ nsLineIterator::CheckLineOrder(PRInt32                  aLine,
   nsLineBox* line = mLines[aLine];
 
   if (!line->mFirstChild) { // empty line
-    *aIsReordered = PR_FALSE;
-    *aFirstVisual = nsnull;
-    *aLastVisual = nsnull;
+    *aIsReordered = false;
+    *aFirstVisual = nullptr;
+    *aLastVisual = nullptr;
     return NS_OK;
   }
 
@@ -669,11 +694,11 @@ nsLineIterator::CheckLineOrder(PRInt32                  aLine,
 #endif // IBMBIDI
 
 NS_IMETHODIMP
-nsLineIterator::FindFrameAt(PRInt32 aLineNumber,
+nsLineIterator::FindFrameAt(int32_t aLineNumber,
                             nscoord aX,
                             nsIFrame** aFrameFound,
-                            PRBool* aXIsBeforeFirstFrame,
-                            PRBool* aXIsAfterLastFrame)
+                            bool* aXIsBeforeFirstFrame,
+                            bool* aXIsAfterLastFrame)
 {
   NS_PRECONDITION(aFrameFound && aXIsBeforeFirstFrame && aXIsAfterLastFrame,
                   "null OUT ptr");
@@ -686,9 +711,9 @@ nsLineIterator::FindFrameAt(PRInt32 aLineNumber,
 
   nsLineBox* line = mLines[aLineNumber];
   if (!line) {
-    *aFrameFound = nsnull;
-    *aXIsBeforeFirstFrame = PR_TRUE;
-    *aXIsAfterLastFrame = PR_FALSE;
+    *aFrameFound = nullptr;
+    *aXIsBeforeFirstFrame = true;
+    *aXIsAfterLastFrame = false;
     return NS_OK;
   }
 
@@ -696,9 +721,9 @@ nsLineIterator::FindFrameAt(PRInt32 aLineNumber,
     return NS_ERROR_FAILURE;
 
   nsIFrame* frame = line->mFirstChild;
-  nsIFrame* closestFromLeft = nsnull;
-  nsIFrame* closestFromRight = nsnull;
-  PRInt32 n = line->GetChildCount();
+  nsIFrame* closestFromLeft = nullptr;
+  nsIFrame* closestFromRight = nullptr;
+  int32_t n = line->GetChildCount();
   while (n--) {
     nsRect rect = frame->GetRect();
     if (rect.width > 0) {
@@ -746,7 +771,7 @@ nsLineIterator::FindFrameAt(PRInt32 aLineNumber,
 }
 
 NS_IMETHODIMP
-nsLineIterator::GetNextSiblingOnLine(nsIFrame*& aFrame, PRInt32 aLineNumber)
+nsLineIterator::GetNextSiblingOnLine(nsIFrame*& aFrame, int32_t aLineNumber)
 {
   aFrame = aFrame->GetNextSibling();
   return NS_OK;
@@ -756,7 +781,7 @@ nsLineIterator::GetNextSiblingOnLine(nsIFrame*& aFrame, PRInt32 aLineNumber)
 
 #ifdef NS_BUILD_REFCNT_LOGGING
 nsFloatCacheList::nsFloatCacheList() :
-  mHead(nsnull)
+  mHead(nullptr)
 {
   MOZ_COUNT_CTOR(nsFloatCacheList);
 }
@@ -777,7 +802,7 @@ nsFloatCacheList::DeleteAll()
     delete c;
     c = next;
   }
-  mHead = nsnull;
+  mHead = nullptr;
 }
 
 nsFloatCache*
@@ -807,8 +832,8 @@ nsFloatCacheList::Append(nsFloatCacheFreeList& aList)
     NS_ASSERTION(!mHead, "Bogus!");
     mHead = aList.mHead;
   }
-  aList.mHead = nsnull;
-  aList.mTail = nsnull;
+  aList.mHead = nullptr;
+  aList.mTail = nullptr;
 }
 
 nsFloatCache*
@@ -828,7 +853,7 @@ nsFloatCache*
 nsFloatCacheList::RemoveAndReturnPrev(nsFloatCache* aElement)
 {
   nsFloatCache* fc = mHead;
-  nsFloatCache* prev = nsnull;
+  nsFloatCache* prev = nullptr;
   while (fc) {
     if (fc == aElement) {
       if (prev) {
@@ -841,14 +866,14 @@ nsFloatCacheList::RemoveAndReturnPrev(nsFloatCache* aElement)
     prev = fc;
     fc = fc->mNext;
   }
-  return nsnull;
+  return nullptr;
 }
 
 //----------------------------------------------------------------------
 
 #ifdef NS_BUILD_REFCNT_LOGGING
 nsFloatCacheFreeList::nsFloatCacheFreeList() :
-  mTail(nsnull)
+  mTail(nullptr)
 {
   MOZ_COUNT_CTOR(nsFloatCacheFreeList);
 }
@@ -873,7 +898,7 @@ nsFloatCacheFreeList::Append(nsFloatCacheList& aList)
     mHead = aList.mHead;
   }
   mTail = aList.Tail();
-  aList.mHead = nsnull;
+  aList.mHead = nullptr;
 }
 
 void
@@ -889,7 +914,7 @@ void
 nsFloatCacheFreeList::DeleteAll()
 {
   nsFloatCacheList::DeleteAll();
-  mTail = nsnull;
+  mTail = nullptr;
 }
 
 nsFloatCache*
@@ -900,12 +925,12 @@ nsFloatCacheFreeList::Alloc(nsIFrame* aFloat)
   nsFloatCache* fc = mHead;
   if (mHead) {
     if (mHead == mTail) {
-      mHead = mTail = nsnull;
+      mHead = mTail = nullptr;
     }
     else {
       mHead = fc->mNext;
     }
-    fc->mNext = nsnull;
+    fc->mNext = nullptr;
   }
   else {
     fc = new nsFloatCache();
@@ -918,7 +943,7 @@ void
 nsFloatCacheFreeList::Append(nsFloatCache* aFloat)
 {
   NS_ASSERTION(!aFloat->mNext, "Bogus!");
-  aFloat->mNext = nsnull;
+  aFloat->mNext = nullptr;
   if (mTail) {
     NS_ASSERTION(!mTail->mNext, "Bogus!");
     mTail->mNext = aFloat;
@@ -933,8 +958,8 @@ nsFloatCacheFreeList::Append(nsFloatCache* aFloat)
 //----------------------------------------------------------------------
 
 nsFloatCache::nsFloatCache()
-  : mFloat(nsnull),
-    mNext(nsnull)
+  : mFloat(nullptr),
+    mNext(nullptr)
 {
   MOZ_COUNT_CTOR(nsFloatCache);
 }
