@@ -1,47 +1,15 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is mozilla.org code.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 1998
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either of the GNU General Public License Version 2 or later (the "GPL"),
- * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 #ifndef nsTableFrame_h__
 #define nsTableFrame_h__
 
+#include "celldata.h"
 #include "nscore.h"
-#include "nsHTMLContainerFrame.h"
+#include "nsContainerFrame.h"
 #include "nsStyleCoord.h"
 #include "nsStyleConsts.h"
-#include "nsITableLayout.h"
 #include "nsTableColFrame.h"
 #include "nsTableColGroupFrame.h"
 #include "nsCellMap.h"
@@ -49,7 +17,9 @@
 #include "nsDisplayList.h"
 
 class nsTableCellFrame;
+class nsTableCellMap;
 class nsTableColFrame;
+class nsColGroupFrame;
 class nsTableRowGroupFrame;
 class nsTableRowFrame;
 class nsTableColGroupFrame;
@@ -58,45 +28,45 @@ class nsStyleContext;
 
 struct nsTableReflowState;
 struct nsStylePosition;
+struct BCPropertyData;
 
-/**
- * Child list name indices
- * @see #GetAdditionalChildListName()
- */
-#define NS_TABLE_FRAME_COLGROUP_LIST_INDEX 0
-#define NS_TABLE_FRAME_OVERFLOW_LIST_INDEX 1
-#define NS_TABLE_FRAME_LAST_LIST_INDEX    NS_TABLE_FRAME_OVERFLOW_LIST_INDEX
-
-static inline PRBool IS_TABLE_CELL(nsIAtom* frameType) {
+static inline bool IS_TABLE_CELL(nsIAtom* frameType) {
   return nsGkAtoms::tableCellFrame == frameType ||
     nsGkAtoms::bcTableCellFrame == frameType;
 }
 
+static inline bool FrameHasBorderOrBackground(nsIFrame* f) {
+  return (f->StyleVisibility()->IsVisible() &&
+          (!f->StyleBackground()->IsTransparent() ||
+           f->StyleDisplay()->mAppearance ||
+           f->StyleBorder()->HasBorder()));
+}
+
 class nsDisplayTableItem : public nsDisplayItem
 {
 public:
   nsDisplayTableItem(nsDisplayListBuilder* aBuilder, nsIFrame* aFrame) : 
       nsDisplayItem(aBuilder, aFrame),
-      mPartHasFixedBackground(PR_FALSE) {}
+      mPartHasFixedBackground(false) {}
 
-  virtual PRBool IsVaryingRelativeToMovingFrame(nsDisplayListBuilder* aBuilder,
+  virtual bool IsVaryingRelativeToMovingFrame(nsDisplayListBuilder* aBuilder,
                                                 nsIFrame* aFrame);
   // With collapsed borders, parts of the collapsed border can extend outside
   // the table part frames, so allow this display element to blow out to our
   // overflow rect. This is also useful for row frames that have spanning
   // cells extending outside them.
-  virtual nsRect GetBounds(nsDisplayListBuilder* aBuilder);
+  virtual nsRect GetBounds(nsDisplayListBuilder* aBuilder, bool* aSnap);
 
   void UpdateForFrameBackground(nsIFrame* aFrame);
 
 private:
-  PRPackedBool mPartHasFixedBackground;
+  bool mPartHasFixedBackground;
 };
 
 class nsAutoPushCurrentTableItem
 {
 public:
-  nsAutoPushCurrentTableItem() : mBuilder(nsnull) {}
+  nsAutoPushCurrentTableItem() : mBuilder(nullptr) {}
   
   void Push(nsDisplayListBuilder* aBuilder, nsDisplayTableItem* aPushItem)
   {
@@ -127,17 +97,15 @@ private:
 
 /* ============================================================================ */
 
-/** nsTableFrame maps the inner portion of a table (everything except captions.)
+/**
+  * nsTableFrame maps the inner portion of a table (everything except captions.)
   * Used as a pseudo-frame within nsTableOuterFrame, it may also be used
   * stand-alone as the top-level frame.
   *
-  * The flowed child list contains row group frames. There is also an additional
-  * named child list:
-  * - "ColGroup-list" which contains the col group frames
-  *
-  * @see nsGkAtoms::colGroupList
+  * The principal child list contains row group frames. There is also an
+  * additional child list, kColGroupList, which contains the col group frames.
   */
-class nsTableFrame : public nsHTMLContainerFrame, public nsITableLayout
+class nsTableFrame : public nsContainerFrame
 {
 public:
   NS_DECL_QUERYFRAME
@@ -164,7 +132,7 @@ public:
 
   // Return true if aParentReflowState.frame or any of its ancestors within
   // the containing table have non-auto height. (e.g. pct or fixed height)
-  static PRBool AncestorsHaveStyleHeight(const nsHTMLReflowState& aParentReflowState);
+  static bool AncestorsHaveStyleHeight(const nsHTMLReflowState& aParentReflowState);
 
   // See if a special height reflow will occur due to having a pct height when
   // the pct height basis may not yet be valid.
@@ -174,11 +142,9 @@ public:
   // height reflow will occur. 
   static void RequestSpecialHeightReflow(const nsHTMLReflowState& aReflowState);
 
-  virtual PRBool IsContainingBlock() const;
-
   static void RePositionViews(nsIFrame* aFrame);
 
-  static PRBool PageBreakAfter(nsIFrame* aSourceFrame,
+  static bool PageBreakAfter(nsIFrame* aSourceFrame,
                                nsIFrame* aNextFrame);
 
   nsPoint GetFirstSectionOrigin(const nsHTMLReflowState& aReflowState) const;
@@ -195,16 +161,17 @@ public:
   /** @see nsIFrame::DidSetStyleContext */
   virtual void DidSetStyleContext(nsStyleContext* aOldStyleContext);
 
-  NS_IMETHOD AppendFrames(nsIAtom*        aListName,
+  NS_IMETHOD AppendFrames(ChildListID     aListID,
                           nsFrameList&    aFrameList);
-  NS_IMETHOD InsertFrames(nsIAtom*        aListName,
+  NS_IMETHOD InsertFrames(ChildListID     aListID,
                           nsIFrame*       aPrevFrame,
                           nsFrameList&    aFrameList);
-  NS_IMETHOD RemoveFrame(nsIAtom*        aListName,
+  NS_IMETHOD RemoveFrame(ChildListID     aListID,
                          nsIFrame*       aOldFrame);
 
   virtual nsMargin GetUsedBorder() const;
   virtual nsMargin GetUsedPadding() const;
+  virtual nsMargin GetUsedMargin() const;
 
   // Get the offset from the border box to the area where the row groups fit
   nsMargin GetChildAreaOffset(const nsHTMLReflowState* aReflowState) const;
@@ -212,11 +179,11 @@ public:
   /** helper method to find the table parent of any table frame object */
   static nsTableFrame* GetTableFrame(nsIFrame* aSourceFrame);
                                  
-  typedef nsresult (* DisplayGenericTablePartTraversal)
+  typedef void (* DisplayGenericTablePartTraversal)
       (nsDisplayListBuilder* aBuilder, nsFrame* aFrame,
        const nsRect& aDirtyRect, const nsDisplayListSet& aLists);
-  static nsresult GenericTraversal(nsDisplayListBuilder* aBuilder, nsFrame* aFrame,
-                                   const nsRect& aDirtyRect, const nsDisplayListSet& aLists);
+  static void GenericTraversal(nsDisplayListBuilder* aBuilder, nsFrame* aFrame,
+                               const nsRect& aDirtyRect, const nsDisplayListSet& aLists);
 
   /**
    * Helper method to handle display common to table frames, rowgroup frames
@@ -229,39 +196,37 @@ public:
    * part's child frames and add their display list items to a
    * display list set.
    */
-  static nsresult DisplayGenericTablePart(nsDisplayListBuilder* aBuilder,
-                                          nsFrame* aFrame,
-                                          const nsRect& aDirtyRect,
-                                          const nsDisplayListSet& aLists,
-                                          nsDisplayTableItem* aDisplayItem,
-                                          DisplayGenericTablePartTraversal aTraversal = GenericTraversal);
+  static void DisplayGenericTablePart(nsDisplayListBuilder* aBuilder,
+                                      nsFrame* aFrame,
+                                      const nsRect& aDirtyRect,
+                                      const nsDisplayListSet& aLists,
+                                      nsDisplayTableItem* aDisplayItem,
+                                      DisplayGenericTablePartTraversal aTraversal = GenericTraversal);
 
   // Return the closest sibling of aPriorChildFrame (including aPriroChildFrame)
   // of type aChildType.
   static nsIFrame* GetFrameAtOrBefore(nsIFrame*       aParentFrame,
                                       nsIFrame*       aPriorChildFrame,
                                       nsIAtom*        aChildType);
-  PRBool IsAutoHeight();
+  bool IsAutoHeight();
   
-  /** @return PR_TRUE if aDisplayType represents a rowgroup of any sort
+  /** @return true if aDisplayType represents a rowgroup of any sort
     * (header, footer, or body)
     */
-  PRBool IsRowGroup(PRInt32 aDisplayType) const;
+  bool IsRowGroup(int32_t aDisplayType) const;
 
   /** Initialize the table frame with a set of children.
     * @see nsIFrame::SetInitialChildList 
     */
-  NS_IMETHOD SetInitialChildList(nsIAtom*        aListName,
+  NS_IMETHOD SetInitialChildList(ChildListID     aListID,
                                  nsFrameList&    aChildList);
 
-  virtual nsFrameList GetChildList(nsIAtom* aListName) const;
+  virtual const nsFrameList& GetChildList(ChildListID aListID) const;
+  virtual void GetChildLists(nsTArray<ChildList>* aLists) const;
 
-  /** @see nsIFrame::GetAdditionalChildListName */
-  virtual nsIAtom* GetAdditionalChildListName(PRInt32 aIndex) const;
-
-  NS_IMETHOD BuildDisplayList(nsDisplayListBuilder*   aBuilder,
-                              const nsRect&           aDirtyRect,
-                              const nsDisplayListSet& aLists);
+  virtual void BuildDisplayList(nsDisplayListBuilder*   aBuilder,
+                                const nsRect&           aDirtyRect,
+                                const nsDisplayListSet& aLists) MOZ_OVERRIDE;
 
   /**
    * Paint the background of the table and its parts (column groups,
@@ -270,7 +235,7 @@ public:
    */
   void PaintTableBorderBackground(nsRenderingContext& aRenderingContext,
                                   const nsRect& aDirtyRect,
-                                  nsPoint aPt, PRUint32 aBGPaintFlags);
+                                  nsPoint aPt, uint32_t aBGPaintFlags);
 
   /** Get the outer half (i.e., the part outside the height and width of
    *  the table) of the largest segment (?) of border-collapsed border on
@@ -304,8 +269,8 @@ public:
 
   friend class nsDelayedCalcBCBorders;
   
-  void SetBCDamageArea(const nsRect& aValue);
-  PRBool BCRecalcNeeded(nsStyleContext* aOldStyleContext,
+  void AddBCDamageArea(const nsIntRect& aValue);
+  bool BCRecalcNeeded(nsStyleContext* aOldStyleContext,
                         nsStyleContext* aNewStyleContext);
   void PaintBCBorders(nsRenderingContext& aRenderingContext,
                       const nsRect&        aDirtyRect);
@@ -321,11 +286,11 @@ public:
   virtual nsSize ComputeSize(nsRenderingContext *aRenderingContext,
                              nsSize aCBSize, nscoord aAvailableWidth,
                              nsSize aMargin, nsSize aBorder, nsSize aPadding,
-                             PRBool aShrinkWrap);
+                             uint32_t aFlags) MOZ_OVERRIDE;
   virtual nsSize ComputeAutoSize(nsRenderingContext *aRenderingContext,
                                  nsSize aCBSize, nscoord aAvailableWidth,
                                  nsSize aMargin, nsSize aBorder,
-                                 nsSize aPadding, PRBool aShrinkWrap);
+                                 nsSize aPadding, bool aShrinkWrap);
   /**
    * A copy of nsFrame::ShrinkWidthToFit that calls a different
    * GetPrefWidth, since tables have two different ones.
@@ -361,9 +326,7 @@ public:
 
   nsFrameList& GetColGroups();
 
-  NS_IMETHOD GetParentStyleContextFrame(nsPresContext* aPresContext,
-                                        nsIFrame**      aProviderFrame,
-                                        PRBool*         aIsChild);
+  virtual nsIFrame* GetParentStyleContextFrame() const;
 
   /**
    * Get the "type" of the frame
@@ -372,21 +335,29 @@ public:
    */
   virtual nsIAtom* GetType() const;
 
+  virtual bool IsFrameOfType(uint32_t aFlags) const
+  {
+    if (aFlags & eSupportsCSSTransforms) {
+      return false;
+    }
+    return nsContainerFrame::IsFrameOfType(aFlags);
+  }
+
 #ifdef DEBUG
   /** @see nsIFrame::GetFrameName */
   NS_IMETHOD GetFrameName(nsAString& aResult) const;
 #endif
 
   /** return the width of the column at aColIndex    */
-  virtual PRInt32 GetColumnWidth(PRInt32 aColIndex);
+  int32_t GetColumnWidth(int32_t aColIndex);
 
   /** helper to get the cell spacing X style value */
-  virtual nscoord GetCellSpacingX();
+  nscoord GetCellSpacingX();
 
   /** helper to get the cell spacing Y style value */
-  virtual nscoord GetCellSpacingY();
+  nscoord GetCellSpacingY();
  
-  virtual nscoord GetBaseline() const;
+  virtual nscoord GetBaseline() const MOZ_OVERRIDE;
   /** return the row span of a cell, taking into account row span magic at the bottom
     * of a table. The row span equals the number of rows spanned by aCell starting at
     * aStartRowIndex, and can be smaller if aStartRowIndex is greater than the row
@@ -398,10 +369,10 @@ public:
     * @return  the row span, correcting for row spans that extend beyond the bottom
     *          of the table.
     */
-  virtual PRInt32  GetEffectiveRowSpan(PRInt32                 aStartRowIndex,
-                                       const nsTableCellFrame& aCell) const;
-  virtual PRInt32  GetEffectiveRowSpan(const nsTableCellFrame& aCell,
-                                       nsCellMap*              aCellMap = nsnull);
+  int32_t  GetEffectiveRowSpan(int32_t                 aStartRowIndex,
+                               const nsTableCellFrame& aCell) const;
+  int32_t  GetEffectiveRowSpan(const nsTableCellFrame& aCell,
+                               nsCellMap*              aCellMap = nullptr);
 
   /** return the col span of a cell, taking into account col span magic at the edge
     * of a table.
@@ -411,19 +382,19 @@ public:
     * @return  the col span, correcting for col spans that extend beyond the edge
     *          of the table.
     */
-  virtual PRInt32  GetEffectiveColSpan(const nsTableCellFrame& aCell,
-                                       nsCellMap*              aCellMap = nsnull) const;
+  int32_t  GetEffectiveColSpan(const nsTableCellFrame& aCell,
+                               nsCellMap*              aCellMap = nullptr) const;
 
   /** indicate whether the row has more than one cell that either originates
     * or is spanned from the rows above
     */
-  PRBool HasMoreThanOneCell(PRInt32 aRowIndex) const;
+  bool HasMoreThanOneCell(int32_t aRowIndex) const;
 
   /** return the column frame associated with aColIndex
-    * returns nsnull if the col frame has not yet been allocated, or if
+    * returns nullptr if the col frame has not yet been allocated, or if
     * aColIndex is out of range
     */
-  nsTableColFrame* GetColFrame(PRInt32 aColIndex) const;
+  nsTableColFrame* GetColFrame(int32_t aColIndex) const;
 
   /** Insert a col frame reference into the colframe cache and adapt the cellmap
     * @param aColFrame    - the column frame
@@ -431,24 +402,24 @@ public:
     *                       colframe cache
     */
   void InsertCol(nsTableColFrame& aColFrame,
-                 PRInt32          aColIndex);
+                 int32_t          aColIndex);
 
   nsTableColGroupFrame* CreateAnonymousColGroupFrame(nsTableColGroupType aType);
 
-  PRInt32 DestroyAnonymousColFrames(PRInt32 aNumFrames);
+  int32_t DestroyAnonymousColFrames(int32_t aNumFrames);
 
   // Append aNumColsToAdd anonymous col frames of type eColAnonymousCell to our
   // last eColGroupAnonymousCell colgroup.  If we have no such colgroup, then
   // create one.
-  void AppendAnonymousColFrames(PRInt32 aNumColsToAdd);
+  void AppendAnonymousColFrames(int32_t aNumColsToAdd);
 
   // Append aNumColsToAdd anonymous col frames of type aColType to
   // aColGroupFrame.  If aAddToTable is true, also call AddColsToTable on the
   // new cols.
   void AppendAnonymousColFrames(nsTableColGroupFrame* aColGroupFrame,
-                                PRInt32               aNumColsToAdd,
+                                int32_t               aNumColsToAdd,
                                 nsTableColType        aColType,
-                                PRBool                aAddToTable);
+                                bool                  aAddToTable);
 
   void MatchCellMapToColCache(nsTableCellMap* aCellMap);
   /** empty the column frame cache */
@@ -456,52 +427,49 @@ public:
 
   void DidResizeColumns();
 
-  virtual void AppendCell(nsTableCellFrame& aCellFrame,
-                          PRInt32           aRowIndex);
+  void AppendCell(nsTableCellFrame& aCellFrame,
+                  int32_t           aRowIndex);
 
-  virtual void InsertCells(nsTArray<nsTableCellFrame*>& aCellFrames,
-                           PRInt32                      aRowIndex,
-                           PRInt32                      aColIndexBefore);
+  void InsertCells(nsTArray<nsTableCellFrame*>& aCellFrames,
+                   int32_t                      aRowIndex,
+                   int32_t                      aColIndexBefore);
 
-  virtual void RemoveCell(nsTableCellFrame* aCellFrame,
-                          PRInt32           aRowIndex);
+  void RemoveCell(nsTableCellFrame* aCellFrame,
+                  int32_t           aRowIndex);
 
   void AppendRows(nsTableRowGroupFrame*       aRowGroupFrame,
-                  PRInt32                     aRowIndex,
+                  int32_t                     aRowIndex,
                   nsTArray<nsTableRowFrame*>& aRowFrames);
 
-  PRInt32 InsertRows(nsTableRowGroupFrame*       aRowGroupFrame,
+  int32_t InsertRows(nsTableRowGroupFrame*       aRowGroupFrame,
                      nsTArray<nsTableRowFrame*>& aFrames,
-                     PRInt32                     aRowIndex,
-                     PRBool                      aConsiderSpans);
+                     int32_t                     aRowIndex,
+                     bool                        aConsiderSpans);
 
-  virtual void RemoveRows(nsTableRowFrame& aFirstRowFrame,
-                          PRInt32          aNumRowsToRemove,
-                          PRBool           aConsiderSpans);
+  void RemoveRows(nsTableRowFrame& aFirstRowFrame,
+                  int32_t          aNumRowsToRemove,
+                  bool             aConsiderSpans);
 
   /** Insert multiple rowgroups into the table cellmap handling
     * @param aRowGroups - iterator that iterates over the rowgroups to insert
     */
   void InsertRowGroups(const nsFrameList::Slice& aRowGroups);
 
-  void InsertColGroups(PRInt32                   aStartColIndex,
+  void InsertColGroups(int32_t                   aStartColIndex,
                        const nsFrameList::Slice& aColgroups);
 
-  virtual void RemoveCol(nsTableColGroupFrame* aColGroupFrame,
-                         PRInt32               aColIndex,
-                         PRBool                aRemoveFromCache,
-                         PRBool                aRemoveFromCellMap);
+  void RemoveCol(nsTableColGroupFrame* aColGroupFrame,
+                 int32_t               aColIndex,
+                 bool                  aRemoveFromCache,
+                 bool                  aRemoveFromCellMap);
 
-  NS_IMETHOD GetIndexByRowAndColumn(PRInt32 aRow, PRInt32 aColumn, PRInt32 *aIndex);
-  NS_IMETHOD GetRowAndColumnByIndex(PRInt32 aIndex, PRInt32 *aRow, PRInt32 *aColumn);
+  bool ColumnHasCellSpacingBefore(int32_t aColIndex) const;
 
-  PRBool ColumnHasCellSpacingBefore(PRInt32 aColIndex) const;
+  bool HasPctCol() const;
+  void SetHasPctCol(bool aValue);
 
-  PRBool HasPctCol() const;
-  void SetHasPctCol(PRBool aValue);
-
-  PRBool HasCellSpanningPctCol() const;
-  void SetHasCellSpanningPctCol(PRBool aValue);
+  bool HasCellSpanningPctCol() const;
+  void SetHasCellSpanningPctCol(bool aValue);
 
   /**
    * To be called on a frame by its parent after setting its size/position and
@@ -515,10 +483,12 @@ public:
    * @param aIsFirstReflow True if the size/position change is due to the
    *                       first reflow of aFrame.
    */
-  static void InvalidateFrame(nsIFrame* aFrame,
-                              const nsRect& aOrigRect,
-                              const nsRect& aOrigVisualOverflow,
-                              PRBool aIsFirstReflow);
+  static void InvalidateTableFrame(nsIFrame* aFrame,
+                                   const nsRect& aOrigRect,
+                                   const nsRect& aOrigVisualOverflow,
+                                   bool aIsFirstReflow);
+
+  virtual bool UpdateOverflow();
 
 protected:
 
@@ -532,12 +502,11 @@ protected:
 
   void InitChildReflowState(nsHTMLReflowState& aReflowState);
 
-  /** implement abstract method on nsHTMLContainerFrame */
-  virtual PRIntn GetSkipSides() const;
+  virtual int GetSkipSides() const MOZ_OVERRIDE;
 
 public:
-  PRBool IsRowInserted() const;
-  void   SetRowInserted(PRBool aValue);
+  bool IsRowInserted() const;
+  void   SetRowInserted(bool aValue);
 
 protected:
     
@@ -565,8 +534,8 @@ protected:
   nscoord GetCollapsedWidth(nsMargin aBorderPadding);
 
   
-  /** Adjust the table for visibilty.collapse set on rowgroups, rows, colgroups
-    * and cols
+  /** Adjust the table for visibility.collapse set on rowgroups, rows,
+    * colgroups and cols
     * @param aDesiredSize    the metrics of the table
     * @param aBorderPadding  the border and padding of the table
     */
@@ -578,6 +547,10 @@ protected:
       mTableLayoutStrategy;
   }
 
+  // Helper for InsertFrames.
+  void HomogenousInsertFrames(ChildListID     aListID,
+                              nsIFrame*       aPrevFrame,
+                              nsFrameList&    aFrameList);
 private:
   /* Handle a row that got inserted during reflow.  aNewHeight is the
      new height of the table after reflow. */
@@ -621,7 +594,7 @@ public:
    * our overflow list or moved directly to our next-in-flow if one exists.
    */
 protected:
-  void PushChildren(const RowGroupArray& aRowGroups, PRInt32 aPushFrom);
+  void PushChildren(const RowGroupArray& aRowGroups, int32_t aPushFrom);
 
 public:
   // put the children frames in the display order (e.g. thead before tbodies
@@ -631,8 +604,8 @@ public:
   // the first one are treated as tbodies instead.
 
   void OrderRowGroups(RowGroupArray& aChildren,
-                      nsTableRowGroupFrame** aHead = nsnull,
-                      nsTableRowGroupFrame** aFoot = nsnull) const;
+                      nsTableRowGroupFrame** aHead = nullptr,
+                      nsTableRowGroupFrame** aFoot = nullptr) const;
 
   // Return the thead, if any
   nsTableRowGroupFrame* GetTHead() const;
@@ -640,35 +613,35 @@ public:
   // Return the tfoot, if any
   nsTableRowGroupFrame* GetTFoot() const;
 
-  // Returns PR_TRUE if there are any cells above the row at
+  // Returns true if there are any cells above the row at
   // aRowIndex and spanning into the row at aRowIndex, the number of
   // effective columns limits the search up to that column
-  PRBool RowIsSpannedInto(PRInt32 aRowIndex, PRInt32 aNumEffCols);
+  bool RowIsSpannedInto(int32_t aRowIndex, int32_t aNumEffCols);
 
-  // Returns PR_TRUE if there is a cell originating in aRowIndex
+  // Returns true if there is a cell originating in aRowIndex
   // which spans into the next row,  the number of effective
   // columns limits the search up to that column
-  PRBool RowHasSpanningCells(PRInt32 aRowIndex, PRInt32 aNumEffCols);
+  bool RowHasSpanningCells(int32_t aRowIndex, int32_t aNumEffCols);
 
 protected:
 
-  PRBool HaveReflowedColGroups() const;
-  void   SetHaveReflowedColGroups(PRBool aValue);
+  bool HaveReflowedColGroups() const;
+  void   SetHaveReflowedColGroups(bool aValue);
 
 public:
-  PRBool IsBorderCollapse() const;
+  bool IsBorderCollapse() const;
 
-  PRBool NeedToCalcBCBorders() const;
-  void SetNeedToCalcBCBorders(PRBool aValue);
+  bool NeedToCalcBCBorders() const;
+  void SetNeedToCalcBCBorders(bool aValue);
 
-  PRBool NeedToCollapse() const;
-  void SetNeedToCollapse(PRBool aValue);
+  bool NeedToCollapse() const;
+  void SetNeedToCollapse(bool aValue);
 
-  PRBool HasZeroColSpans() const;
-  void SetHasZeroColSpans(PRBool aValue);
+  bool HasZeroColSpans() const;
+  void SetHasZeroColSpans(bool aValue);
 
-  PRBool NeedColSpanExpansion() const;
-  void SetNeedColSpanExpansion(PRBool aValue);
+  bool NeedColSpanExpansion() const;
+  void SetNeedColSpanExpansion(bool aValue);
 
   /** The GeometryDirty bit is similar to the NS_FRAME_IS_DIRTY frame
     * state bit, which implies that all descendants are dirty.  The
@@ -676,22 +649,22 @@ public:
     * dirty, but resizing optimizations should still apply to the
     * contents of the individual cells.
     */
-  void SetGeometryDirty() { mBits.mGeometryDirty = PR_TRUE; }
-  void ClearGeometryDirty() { mBits.mGeometryDirty = PR_FALSE; }
-  PRBool IsGeometryDirty() const { return mBits.mGeometryDirty; }
+  void SetGeometryDirty() { mBits.mGeometryDirty = true; }
+  void ClearGeometryDirty() { mBits.mGeometryDirty = false; }
+  bool IsGeometryDirty() const { return mBits.mGeometryDirty; }
 
   /** Get the cell map for this table frame.  It is not always mCellMap.
     * Only the firstInFlow has a legit cell map
     */
-  virtual nsTableCellMap* GetCellMap() const;
+  nsTableCellMap* GetCellMap() const;
 
   /** Iterate over the row groups and adjust the row indices of all rows 
     * whose index is >= aRowIndex.  
     * @param aRowIndex   - start adjusting with this index
     * @param aAdjustment - shift the row index by this amount
     */
-  void AdjustRowIndices(PRInt32 aRowIndex,
-                        PRInt32 aAdjustment);
+  void AdjustRowIndices(int32_t aRowIndex,
+                        int32_t aAdjustment);
 
   /** Reset the rowindices of all rows as they might have changed due to 
     * rowgroup reordering, exclude new row group frames that show in the
@@ -703,74 +676,63 @@ public:
 
   nsTArray<nsTableColFrame*>& GetColCache();
 
+
 protected:
 
-  void SetBorderCollapse(PRBool aValue);
+  void SetBorderCollapse(bool aValue);
 
+  BCPropertyData* GetBCProperty(bool aCreateIfNecessary = false) const;
+  void SetFullBCDamageArea();
   void CalcBCBorders();
 
-  void ExpandBCDamageArea(nsRect& aRect) const;
+  void ExpandBCDamageArea(nsIntRect& aRect) const;
 
   void SetColumnDimensions(nscoord         aHeight,
                            const nsMargin& aReflowState);
 
-  PRInt32 CollectRows(nsIFrame*                   aFrame,
+  int32_t CollectRows(nsIFrame*                   aFrame,
                       nsTArray<nsTableRowFrame*>& aCollection);
 
 public: /* ----- Cell Map public methods ----- */
 
-  PRInt32 GetStartRowIndex(nsTableRowGroupFrame* aRowGroupFrame);
+  int32_t GetStartRowIndex(nsTableRowGroupFrame* aRowGroupFrame);
 
   /** returns the number of rows in this table.
     */
-  PRInt32 GetRowCount () const
+  int32_t GetRowCount () const
   {
     return GetCellMap()->GetRowCount();
   }
 
   /** returns the number of columns in this table after redundant columns have been removed 
     */
-  PRInt32 GetEffectiveColCount() const;
+  int32_t GetEffectiveColCount() const;
 
   /* return the col count including dead cols */
-  PRInt32 GetColCount () const
+  int32_t GetColCount () const
   {
     return GetCellMap()->GetColCount();
   }
 
   // return the last col index which isn't of type eColAnonymousCell
-  PRInt32 GetIndexOfLastRealCol();
+  int32_t GetIndexOfLastRealCol();
 
-  /** returns PR_TRUE if table-layout:auto  */
-  virtual PRBool IsAutoLayout();
-
-  /*---------------- nsITableLayout methods ------------------------*/
-  
-  /** Get the cell and associated data for a table cell from the frame's cellmap */
-  NS_IMETHOD GetCellDataAt(PRInt32 aRowIndex, PRInt32 aColIndex, 
-                           nsIDOMElement* &aCell,   //out params
-                           PRInt32& aStartRowIndex, PRInt32& aStartColIndex, 
-                           PRInt32& aRowSpan, PRInt32& aColSpan,
-                           PRInt32& aActualRowSpan, PRInt32& aActualColSpan,
-                           PRBool& aIsSelected);
-
-  /** Get the number of rows and column for a table from the frame's cellmap 
-    *  Some rows may not have enough cells (the number returned is the maximum possible),
-    *  which displays as a ragged-right edge table
-    */
-  NS_IMETHOD GetTableSize(PRInt32& aRowCount, PRInt32& aColCount);
-
-  /*------------end of nsITableLayout methods -----------------------*/
+  /** returns true if table-layout:auto  */
+  bool IsAutoLayout();
 
 public:
  
 #ifdef DEBUG
-  void Dump(PRBool          aDumpRows,
-            PRBool          aDumpCols, 
-            PRBool          aDumpCellMap);
+  void Dump(bool            aDumpRows,
+            bool            aDumpCols, 
+            bool            aDumpCellMap);
 #endif
 
 protected:
+  /**
+   * Helper method for RemoveFrame.
+   */
+  void DoRemoveFrame(ChildListID aListID, nsIFrame* aOldFrame);
 #ifdef DEBUG
   void DumpRowGroup(nsIFrame* aChildFrame);
 #endif
@@ -778,18 +740,18 @@ protected:
   nsAutoTArray<nsTableColFrame*, 8> mColFrames;
 
   struct TableBits {
-    PRUint32 mHaveReflowedColGroups:1; // have the col groups gotten their initial reflow
-    PRUint32 mHasPctCol:1;             // does any cell or col have a pct width
-    PRUint32 mCellSpansPctCol:1;       // does any cell span a col with a pct width (or containing a cell with a pct width)
-    PRUint32 mIsBorderCollapse:1;      // border collapsing model vs. separate model
-    PRUint32 mRowInserted:1;
-    PRUint32 mNeedToCalcBCBorders:1;
-    PRUint32 mGeometryDirty:1;
-    PRUint32 mLeftContBCBorder:8;
-    PRUint32 mNeedToCollapse:1;    // rows, cols that have visibility:collapse need to be collapsed
-    PRUint32 mHasZeroColSpans:1;
-    PRUint32 mNeedColSpanExpansion:1;
-    PRUint32 mResizedColumns:1;        // have we resized columns since last reflow?
+    uint32_t mHaveReflowedColGroups:1; // have the col groups gotten their initial reflow
+    uint32_t mHasPctCol:1;             // does any cell or col have a pct width
+    uint32_t mCellSpansPctCol:1;       // does any cell span a col with a pct width (or containing a cell with a pct width)
+    uint32_t mIsBorderCollapse:1;      // border collapsing model vs. separate model
+    uint32_t mRowInserted:1;
+    uint32_t mNeedToCalcBCBorders:1;
+    uint32_t mGeometryDirty:1;
+    uint32_t mLeftContBCBorder:8;
+    uint32_t mNeedToCollapse:1;    // rows, cols that have visibility:collapse need to be collapsed
+    uint32_t mHasZeroColSpans:1;
+    uint32_t mNeedColSpanExpansion:1;
+    uint32_t mResizedColumns:1;        // have we resized columns since last reflow?
   } mBits;
 
   nsTableCellMap*         mCellMap;            // maintains the relationships between rows, cols, and cells
@@ -798,81 +760,81 @@ protected:
 };
 
 
-inline PRBool nsTableFrame::IsRowGroup(PRInt32 aDisplayType) const
+inline bool nsTableFrame::IsRowGroup(int32_t aDisplayType) const
 {
-  return PRBool((NS_STYLE_DISPLAY_TABLE_HEADER_GROUP == aDisplayType) ||
+  return bool((NS_STYLE_DISPLAY_TABLE_HEADER_GROUP == aDisplayType) ||
                 (NS_STYLE_DISPLAY_TABLE_FOOTER_GROUP == aDisplayType) ||
                 (NS_STYLE_DISPLAY_TABLE_ROW_GROUP    == aDisplayType));
 }
 
-inline void nsTableFrame::SetHaveReflowedColGroups(PRBool aValue)
+inline void nsTableFrame::SetHaveReflowedColGroups(bool aValue)
 {
   mBits.mHaveReflowedColGroups = aValue;
 }
 
-inline PRBool nsTableFrame::HaveReflowedColGroups() const
+inline bool nsTableFrame::HaveReflowedColGroups() const
 {
-  return (PRBool)mBits.mHaveReflowedColGroups;
+  return (bool)mBits.mHaveReflowedColGroups;
 }
 
-inline PRBool nsTableFrame::HasPctCol() const
+inline bool nsTableFrame::HasPctCol() const
 {
-  return (PRBool)mBits.mHasPctCol;
+  return (bool)mBits.mHasPctCol;
 }
 
-inline void nsTableFrame::SetHasPctCol(PRBool aValue)
+inline void nsTableFrame::SetHasPctCol(bool aValue)
 {
   mBits.mHasPctCol = (unsigned)aValue;
 }
 
-inline PRBool nsTableFrame::HasCellSpanningPctCol() const
+inline bool nsTableFrame::HasCellSpanningPctCol() const
 {
-  return (PRBool)mBits.mCellSpansPctCol;
+  return (bool)mBits.mCellSpansPctCol;
 }
 
-inline void nsTableFrame::SetHasCellSpanningPctCol(PRBool aValue)
+inline void nsTableFrame::SetHasCellSpanningPctCol(bool aValue)
 {
   mBits.mCellSpansPctCol = (unsigned)aValue;
 }
 
-inline PRBool nsTableFrame::IsRowInserted() const
+inline bool nsTableFrame::IsRowInserted() const
 {
-  return (PRBool)mBits.mRowInserted;
+  return (bool)mBits.mRowInserted;
 }
 
-inline void nsTableFrame::SetRowInserted(PRBool aValue)
+inline void nsTableFrame::SetRowInserted(bool aValue)
 {
   mBits.mRowInserted = (unsigned)aValue;
 }
 
-inline void nsTableFrame::SetNeedToCollapse(PRBool aValue)
+inline void nsTableFrame::SetNeedToCollapse(bool aValue)
 {
   static_cast<nsTableFrame*>(GetFirstInFlow())->mBits.mNeedToCollapse = (unsigned)aValue;
 }
 
-inline PRBool nsTableFrame::NeedToCollapse() const
+inline bool nsTableFrame::NeedToCollapse() const
 {
-  return (PRBool) static_cast<nsTableFrame*>(GetFirstInFlow())->mBits.mNeedToCollapse;
+  return (bool) static_cast<nsTableFrame*>(GetFirstInFlow())->mBits.mNeedToCollapse;
 }
 
-inline void nsTableFrame::SetHasZeroColSpans(PRBool aValue)
+inline void nsTableFrame::SetHasZeroColSpans(bool aValue)
 {
   mBits.mHasZeroColSpans = (unsigned)aValue;
 }
 
-inline PRBool nsTableFrame::HasZeroColSpans() const
+inline bool nsTableFrame::HasZeroColSpans() const
 {
-  return (PRBool)mBits.mHasZeroColSpans;
+  return (bool)mBits.mHasZeroColSpans;
 }
 
-inline void nsTableFrame::SetNeedColSpanExpansion(PRBool aValue)
+inline void nsTableFrame::SetNeedColSpanExpansion(bool aValue)
 {
   mBits.mNeedColSpanExpansion = (unsigned)aValue;
 }
 
-inline PRBool nsTableFrame::NeedColSpanExpansion() const
+inline bool nsTableFrame::NeedColSpanExpansion() const
 {
-  return (PRBool)mBits.mNeedColSpanExpansion;
+  return (bool)mBits.mNeedColSpanExpansion;
 }
 
 
@@ -886,22 +848,22 @@ inline nsTArray<nsTableColFrame*>& nsTableFrame::GetColCache()
   return mColFrames;
 }
 
-inline PRBool nsTableFrame::IsBorderCollapse() const
+inline bool nsTableFrame::IsBorderCollapse() const
 {
-  return (PRBool)mBits.mIsBorderCollapse;
+  return (bool)mBits.mIsBorderCollapse;
 }
 
-inline void nsTableFrame::SetBorderCollapse(PRBool aValue) 
+inline void nsTableFrame::SetBorderCollapse(bool aValue) 
 {
   mBits.mIsBorderCollapse = aValue;
 }
 
-inline PRBool nsTableFrame::NeedToCalcBCBorders() const
+inline bool nsTableFrame::NeedToCalcBCBorders() const
 {
-  return (PRBool)mBits.mNeedToCalcBCBorders;
+  return (bool)mBits.mNeedToCalcBCBorders;
 }
 
-inline void nsTableFrame::SetNeedToCalcBCBorders(PRBool aValue)
+inline void nsTableFrame::SetNeedToCalcBCBorders(bool aValue)
 {
   mBits.mNeedToCalcBCBorders = (unsigned)aValue;
 }
@@ -909,7 +871,7 @@ inline void nsTableFrame::SetNeedToCalcBCBorders(PRBool aValue)
 inline nscoord
 nsTableFrame::GetContinuousLeftBCBorderWidth() const
 {
-  PRInt32 aPixelsToTwips = nsPresContext::AppUnitsPerCSSPixel();
+  int32_t aPixelsToTwips = nsPresContext::AppUnitsPerCSSPixel();
   return BC_BORDER_RIGHT_HALF_COORD(aPixelsToTwips, mBits.mLeftContBCBorder);
 }
 
@@ -925,24 +887,24 @@ public:
   nsTableIterator(nsFrameList& aSource);
   nsIFrame* First();
   nsIFrame* Next();
-  PRBool    IsLeftToRight();
-  PRInt32   Count();
+  bool      IsLeftToRight();
+  int32_t   Count();
 
 protected:
   void Init(nsIFrame* aFirstChild);
-  PRBool    mLeftToRight;
+  bool      mLeftToRight;
   nsIFrame* mFirstListChild;
   nsIFrame* mFirstChild;
   nsIFrame* mCurrentChild;
-  PRInt32   mCount;
+  int32_t   mCount;
 };
 
 #define ABORT0() \
-{NS_ASSERTION(PR_FALSE, "CellIterator program error"); \
+{NS_ASSERTION(false, "CellIterator program error"); \
 return;}
 
 #define ABORT1(aReturn) \
-{NS_ASSERTION(PR_FALSE, "CellIterator program error"); \
+{NS_ASSERTION(false, "CellIterator program error"); \
 return aReturn;} 
 
 #endif
