@@ -1,43 +1,14 @@
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Initial Developer of the Original Code is
- * CSIRO
- * Portions created by the Initial Developer are Copyright (C) 2007
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s): Michael Martin
- *                 Chris Double (chris.double@double.co.nz)
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** *
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/.
  */
 #include <pthread.h>
 #include <stdlib.h>
 #include <alsa/asoundlib.h>
 #include "sydney_audio.h"
 
+#define ALSA_PA_PLUGIN "ALSA <-> PulseAudio PCM I/O Plugin"
+
 /* ALSA implementation based heavily on sydney_audio_mac.c */
 
 pthread_mutex_t sa_alsa_mutex = PTHREAD_MUTEX_INITIALIZER;
@@ -174,9 +145,8 @@ sa_stream_open(sa_stream_t *s) {
   snd_output_buffer_open(&out);
   snd_pcm_dump(s->output_unit, out);
   bufsz = snd_output_buffer_string(out, &buf);
-  if (strncmp(buf, "ALSA <-> PulseAudio PCM I/O Plugin", bufsz) > 0 ) {
-    s->pulseaudio = 1;
-  }
+  s->pulseaudio = bufsz >= strlen(ALSA_PA_PLUGIN) &&
+                  strncmp(buf, ALSA_PA_PLUGIN, strlen(ALSA_PA_PLUGIN)) == 0;
   snd_output_close(out);
 
   snd_pcm_hw_params_alloca(&hwparams);
@@ -190,7 +160,7 @@ sa_stream_open(sa_stream_t *s) {
 
 
 int
-sa_stream_get_min_write(sa_stream_t *s, size_t *samples) {
+sa_stream_get_min_write(sa_stream_t *s, size_t *size) {
   int r;
   snd_pcm_uframes_t threshold;
   snd_pcm_sw_params_t* swparams;
@@ -200,9 +170,10 @@ sa_stream_get_min_write(sa_stream_t *s, size_t *samples) {
   snd_pcm_sw_params_alloca(&swparams);
   snd_pcm_sw_params_current(s->output_unit, swparams);
   r = snd_pcm_sw_params_get_start_threshold(swparams, &threshold);
-  if (r < 0)
+  if (r < 0) {
     return SA_ERROR_NO_INIT;
-  *samples = threshold;
+  }
+  *size = snd_pcm_frames_to_bytes(s->output_unit, threshold);
 
   return SA_SUCCESS;
 }
@@ -386,12 +357,13 @@ sa_stream_drain(sa_stream_t *s)
   if (snd_pcm_state(s->output_unit) == SND_PCM_STATE_PREPARED) {
     size_t min_samples = 0;
     size_t min_bytes = 0;
+    void *buf;
 
     if (sa_stream_get_min_write(s, &min_samples) < 0)
       return SA_ERROR_SYSTEM;
     min_bytes = snd_pcm_frames_to_bytes(s->output_unit, min_samples);    
 
-    void* buf = malloc(min_bytes);
+    buf = malloc(min_bytes);
     if (!buf)
       return SA_ERROR_SYSTEM;
     memset(buf, 0, min_bytes);
@@ -571,6 +543,7 @@ UNSUPPORTED(int sa_stream_write_ni(sa_stream_t *s, unsigned int channel, const v
 UNSUPPORTED(int sa_stream_pwrite(sa_stream_t *s, const void *data, size_t nbytes, int64_t offset, sa_seek_t whence))
 UNSUPPORTED(int sa_stream_pwrite_ni(sa_stream_t *s, unsigned int channel, const void *data, size_t nbytes, int64_t offset, sa_seek_t whence))
 UNSUPPORTED(int sa_stream_get_read_size(sa_stream_t *s, size_t *size))
+UNSUPPORTED(int sa_stream_set_stream_type(sa_stream_t *s, const sa_stream_type_t stream_type))
 
 const char *sa_strerror(int code) { return NULL; }
 
