@@ -1,40 +1,8 @@
-# -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
-# ***** BEGIN LICENSE BLOCK *****
-# Version: MPL 1.1/GPL 2.0/LGPL 2.1
-#
-# The contents of this file are subject to the Mozilla Public License Version
-# 1.1 (the "License"); you may not use this file except in compliance with
-# the License. You may obtain a copy of the License at
-# http://www.mozilla.org/MPL/
-#
-# Software distributed under the License is distributed on an "AS IS" basis,
-# WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
-# for the specific language governing rights and limitations under the
-# License.
-#
-# The Original Code is The Update Service.
-#
-# The Initial Developer of the Original Code is Ben Goodger.
-# Portions created by the Initial Developer are Copyright (C) 2004
-# the Initial Developer. All Rights Reserved.
-#
-# Contributor(s):
-#   Ben Goodger <ben@bengoodger.com>
-#   Robert Strong <robert.bugzilla@gmail.com>
-#
-# Alternatively, the contents of this file may be used under the terms of
-# either the GNU General Public License Version 2 or later (the "GPL"), or
-# the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
-# in which case the provisions of the GPL or the LGPL are applicable instead
-# of those above. If you wish to allow use of your version of this file only
-# under the terms of either the GPL or the LGPL, and not to allow others to
-# use your version of this file under the terms of the MPL, indicate your
-# decision by deleting the provisions above and replace them with the notice
-# and other provisions required by the GPL or the LGPL. If you do not delete
-# the provisions above, a recipient may use your version of this file under
-# the terms of any one of the MPL, the GPL or the LGPL.
-#
-# ***** END LICENSE BLOCK *****
+// -*- Mode: Java; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
+
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 // This UI is only opened from the Extension Manager when the app is upgraded.
 
@@ -42,9 +10,16 @@
 
 const PREF_UPDATE_EXTENSIONS_ENABLED            = "extensions.update.enabled";
 const PREF_XPINSTALL_ENABLED                    = "xpinstall.enabled";
+const PREF_EM_HOTFIX_ID                         = "extensions.hotfix.id";
 
 Components.utils.import("resource://gre/modules/Services.jsm");
 Components.utils.import("resource://gre/modules/AddonManager.jsm");
+Components.utils.import("resource://gre/modules/AddonRepository.jsm");
+
+
+var gInteruptable = true;
+var gPendingClose = false;
+
 
 var gUpdateWizard = {
   // When synchronizing app compatibility info this contains all installed
@@ -61,7 +36,7 @@ var gUpdateWizard = {
   xpinstallEnabled: true,
   xpinstallLocked: false,
 
-  init: function ()
+  init: function gUpdateWizard_init()
   {
     this.inactiveAddonIDs = window.arguments[0];
 
@@ -85,13 +60,13 @@ var gUpdateWizard = {
       document.documentElement.currentPage = document.getElementById("versioninfo");
   },
 
-  onWizardFinish: function ()
+  onWizardFinish: function gUpdateWizard_onWizardFinish ()
   {
     if (this.shouldSuggestAutoChecking)
       Services.prefs.setBoolPref(PREF_UPDATE_EXTENSIONS_ENABLED, this.shouldAutoCheck);
   },
 
-  _setUpButton: function (aButtonID, aButtonKey, aDisabled)
+  _setUpButton: function gUpdateWizard_setUpButton(aButtonID, aButtonKey, aDisabled)
   {
     var strings = document.getElementById("updateStrings");
     var button = document.documentElement.getButton(aButtonID);
@@ -106,7 +81,7 @@ var gUpdateWizard = {
     button.disabled = aDisabled;
   },
 
-  setButtonLabels: function (aBackButton, aBackButtonIsDisabled,
+  setButtonLabels: function gUpdateWizard_setButtonLabels(aBackButton, aBackButtonIsDisabled,
                              aNextButton, aNextButtonIsDisabled,
                              aCancelButton, aCancelButtonIsDisabled)
   {
@@ -119,14 +94,27 @@ var gUpdateWizard = {
   // Update Errors
   errorItems: [],
 
-  checkForErrors: function (aElementIDToShow)
+  checkForErrors: function gUpdateWizard_checkForErrors(aElementIDToShow)
   {
     if (this.errorItems.length > 0)
       document.getElementById(aElementIDToShow).hidden = false;
   },
 
-  onWizardClose: function (aEvent)
+  onWizardClose: function gUpdateWizard_onWizardClose(aEvent)
   {
+    return this.onWizardCancel();
+  },
+
+  onWizardCancel: function gUpdateWizard_onWizardCancel()
+  {
+    if (!gInteruptable) {
+      gPendingClose = true;
+      this._setUpButton("back", null, true);
+      this._setUpButton("next", null, true);
+      this._setUpButton("cancel", null, true);
+      return false;
+    }
+
     if (gInstallingPage.installing) {
       gInstallingPage.cancelInstalls();
       return false;
@@ -136,13 +124,13 @@ var gUpdateWizard = {
 };
 
 var gOfflinePage = {
-  onPageAdvanced: function ()
+  onPageAdvanced: function gOfflinePage_onPageAdvanced()
   {
     Services.io.offline = false;
     return true;
   },
 
-  toggleOffline: function ()
+  toggleOffline: function gOfflinePage_toggleOffline()
   {
     var nextbtn = document.documentElement.getButton("next");
     nextbtn.disabled = !nextbtn.disabled;
@@ -152,30 +140,49 @@ var gOfflinePage = {
 var gVersionInfoPage = {
   _completeCount: 0,
   _totalCount: 0,
-  onPageShow: function ()
+  onPageShow: function gVersionInfoPage_onPageShow()
   {
     gUpdateWizard.setButtonLabels(null, true,
                                   "nextButtonText", true,
                                   "cancelButtonText", false);
 
+    try {
+      var hotfixID = Services.prefs.getCharPref(PREF_EM_HOTFIX_ID);
+    }
+    catch (e) { }
+
     // Retrieve all add-ons in order to sync their app compatibility information
-    AddonManager.getAllAddons(function(aAddons) {
-      gUpdateWizard.addons = aAddons.filter(function(a) {
-        return a.type != "plugin";
+    AddonManager.getAllAddons(function gVersionInfoPage_getAllAddons(aAddons) {
+      gUpdateWizard.addons = aAddons.filter(function gVersionInfoPage_filterAddons(a) {
+        return a.type != "plugin" && a.id != hotfixID;
       });
 
       gVersionInfoPage._totalCount = gUpdateWizard.addons.length;
 
-      gUpdateWizard.addons.forEach(function(aAddon) {
-        aAddon.findUpdates(gVersionInfoPage, AddonManager.UPDATE_WHEN_NEW_APP_INSTALLED);
-      }, this);
+      // Ensure compatibility overrides are up to date before checking for
+      // individual addon updates.
+      let ids = [addon.id for each (addon in gUpdateWizard.addons)];
+
+      gInteruptable = false;
+      AddonRepository.repopulateCache(ids, function gVersionInfoPage_repolulateCache() {
+        AddonManagerPrivate.updateAddonRepositoryData(function gVersionInfoPage_updateAddonRepoData() {
+          gInteruptable = true;
+          if (gPendingClose) {
+            window.close();
+            return;
+          }
+
+          for (let addon of gUpdateWizard.addons)
+            addon.findUpdates(gVersionInfoPage, AddonManager.UPDATE_WHEN_NEW_APP_INSTALLED);
+        });
+      });
     });
   },
 
-  onAllUpdatesFinished: function() {
+  onAllUpdatesFinished: function gVersionInfoPage_onAllUpdatesFinished() {
     // Filter out any add-ons that were disabled before the application was
     // upgraded or are already compatible
-    gUpdateWizard.addons = gUpdateWizard.addons.filter(function(a) {
+    gUpdateWizard.addons = gUpdateWizard.addons.filter(function onAllUpdatesFinished_filterAddons(a) {
       return a.appDisabled && gUpdateWizard.inactiveAddonIDs.indexOf(a.id) < 0;
     });
 
@@ -194,7 +201,7 @@ var gVersionInfoPage = {
 
   /////////////////////////////////////////////////////////////////////////////
   // UpdateListener
-  onUpdateFinished: function(aAddon, status) {
+  onUpdateFinished: function gVersionInfoPage_onUpdateFinished(aAddon, status) {
     // If the add-on is now active then it won't have been disabled by startup
     if (aAddon.active)
       AddonManagerPrivate.removeStartupChange("disabled", aAddon.id);
@@ -206,9 +213,9 @@ var gVersionInfoPage = {
 
     // Update the status text and progress bar
     var updateStrings = document.getElementById("updateStrings");
-    var status = document.getElementById("versioninfo.status");
+    var statusElt = document.getElementById("versioninfo.status");
     var statusString = updateStrings.getFormattedString("statusPrefix", [aAddon.name]);
-    status.setAttribute("value", statusString);
+    statusElt.setAttribute("value", statusString);
 
     // Update the status text and progress bar
     var progress = document.getElementById("versioninfo.progress");
@@ -221,7 +228,7 @@ var gVersionInfoPage = {
 };
 
 var gMismatchPage = {
-  onPageShow: function ()
+  onPageShow: function gMismatchPage_onPageShow()
   {
     gUpdateWizard.setButtonLabels(null, true,
                                   "mismatchCheckNow", false,
@@ -229,18 +236,18 @@ var gMismatchPage = {
     document.documentElement.getButton("next").focus();
 
     var incompatible = document.getElementById("mismatch.incompatible");
-    gUpdateWizard.addons.forEach(function(aAddon) {
+    for (let addon of gUpdateWizard.addons) {
       var listitem = document.createElement("listitem");
-      listitem.setAttribute("label", aAddon.name + " " + aAddon.version);
+      listitem.setAttribute("label", addon.name + " " + addon.version);
       incompatible.appendChild(listitem);
-    });
+    }
   }
 };
 
 var gUpdatePage = {
   _totalCount: 0,
   _completeCount: 0,
-  onPageShow: function ()
+  onPageShow: function gUpdatePage_onPageShow()
   {
     if (!gUpdateWizard.xpinstallEnabled && gUpdateWizard.xpinstallLocked) {
       document.documentElement.currentPage = document.getElementById("adminDisabled");
@@ -255,12 +262,11 @@ var gUpdatePage = {
     gUpdateWizard.errorItems = [];
 
     this._totalCount = gUpdateWizard.addons.length;
-    gUpdateWizard.addons.forEach(function(aAddon) {
-      aAddon.findUpdates(this, AddonManager.UPDATE_WHEN_NEW_APP_INSTALLED);
-    }, this);
+    for (let addon of gUpdateWizard.addons)
+      addon.findUpdates(this, AddonManager.UPDATE_WHEN_NEW_APP_INSTALLED);
   },
 
-  onAllUpdatesFinished: function() {
+  onAllUpdatesFinished: function gUpdatePage_onAllUpdatesFinished() {
     var nextPage = document.getElementById("noupdates");
     if (gUpdateWizard.addonsToUpdate.length > 0)
       nextPage = document.getElementById("found");
@@ -269,11 +275,11 @@ var gUpdatePage = {
 
   /////////////////////////////////////////////////////////////////////////////
   // UpdateListener
-  onUpdateAvailable: function(aAddon, aInstall) {
+  onUpdateAvailable: function gUpdatePage_onUpdateAvailable(aAddon, aInstall) {
     gUpdateWizard.addonsToUpdate.push(aInstall);
   },
 
-  onUpdateFinished: function(aAddon, status) {
+  onUpdateFinished: function gUpdatePage_onUpdateFinished(aAddon, status) {
     if (status != AddonManager.UPDATE_STATUS_NO_ERROR)
       gUpdateWizard.errorItems.push(aAddon);
 
@@ -281,9 +287,9 @@ var gUpdatePage = {
 
     // Update the status text and progress bar
     var updateStrings = document.getElementById("updateStrings");
-    var status = document.getElementById("checking.status");
+    var statusElt = document.getElementById("checking.status");
     var statusString = updateStrings.getFormattedString("statusPrefix", [aAddon.name]);
-    status.setAttribute("value", statusString);
+    statusElt.setAttribute("value", statusString);
 
     var progress = document.getElementById("checking.progress");
     progress.value = Math.ceil((this._completeCount / this._totalCount) * 100);
@@ -294,7 +300,7 @@ var gUpdatePage = {
 };
 
 var gFoundPage = {
-  onPageShow: function ()
+  onPageShow: function gFoundPage_onPageShow()
   {
     gUpdateWizard.setButtonLabels(null, true,
                                   "installButtonText", false,
@@ -302,12 +308,12 @@ var gFoundPage = {
 
     var foundUpdates = document.getElementById("found.updates");
     var itemCount = gUpdateWizard.addonsToUpdate.length;
-    gUpdateWizard.addonsToUpdate.forEach(function(aInstall) {
-      var listItem = foundUpdates.appendItem(aInstall.name + " " + aInstall.version);
+    for (let install of gUpdateWizard.addonsToUpdate) {
+      let listItem = foundUpdates.appendItem(install.name + " " + install.version);
       listItem.setAttribute("type", "checkbox");
       listItem.setAttribute("checked", "true");
-      listItem.install = aInstall;
-    });
+      listItem.install = install;
+    }
 
     if (!gUpdateWizard.xpinstallEnabled) {
       document.getElementById("xpinstallDisabledAlert").hidden = false;
@@ -320,7 +326,7 @@ var gFoundPage = {
     }
   },
 
-  toggleXPInstallEnable: function(aEvent)
+  toggleXPInstallEnable: function gFoundPage_toggleXPInstallEnable(aEvent)
   {
     var enabled = aEvent.target.checked;
     gUpdateWizard.xpinstallEnabled = enabled;
@@ -330,7 +336,7 @@ var gFoundPage = {
     this.updateNextButton();
   },
 
-  updateNextButton: function ()
+  updateNextButton: function gFoundPage_updateNextButton()
   {
     if (!gUpdateWizard.xpinstallEnabled) {
       document.documentElement.getButton("next").disabled = true;
@@ -340,8 +346,8 @@ var gFoundPage = {
     var oneChecked = false;
     var foundUpdates = document.getElementById("found.updates");
     var updates = foundUpdates.getElementsByTagName("listitem");
-    for (var i = 0; i < updates.length; ++i) {
-      if (!updates[i].checked)
+    for (let update of updates) {
+      if (!update.checked)
         continue;
       oneChecked = true;
       break;
@@ -362,7 +368,7 @@ var gInstallingPage = {
   _currentInstall   : -1,
   _installing       : false,
 
-  onPageShow: function ()
+  onPageShow: function gInstallingPage_onPageShow()
   {
     gUpdateWizard.setButtonLabels(null, true,
                                   "nextButtonText", true,
@@ -371,10 +377,10 @@ var gInstallingPage = {
 
     var foundUpdates = document.getElementById("found.updates");
     var updates = foundUpdates.getElementsByTagName("listitem");
-    for (var i = 0; i < updates.length; ++i) {
-      if (!updates[i].checked)
+    for (let update of updates) {
+      if (!update.checked)
         continue;
-      this._installs.push(updates[i].install);
+      this._installs.push(update.install);
     }
 
     this._strings = document.getElementById("updateStrings");
@@ -382,7 +388,7 @@ var gInstallingPage = {
     this.startNextInstall();
   },
 
-  startNextInstall: function() {
+  startNextInstall: function gInstallingPage_startNextInstall() {
     if (this._currentInstall >= 0) {
       this._installs[this._currentInstall].removeListener(this);
     }
@@ -401,50 +407,50 @@ var gInstallingPage = {
     this._installs[this._currentInstall].install();
   },
 
-  cancelInstalls: function() {
+  cancelInstalls: function gInstallingPage_cancelInstalls() {
     this._installs[this._currentInstall].removeListener(this);
     this._installs[this._currentInstall].cancel();
   },
 
   /////////////////////////////////////////////////////////////////////////////
   // InstallListener
-  onDownloadStarted: function(aInstall) {
+  onDownloadStarted: function gInstallingPage_onDownloadStarted(aInstall) {
     var strings = document.getElementById("updateStrings");
     var label = strings.getFormattedString("downloadingPrefix", [aInstall.name]);
     var actionItem = document.getElementById("actionItem");
     actionItem.value = label;
   },
 
-  onDownloadProgress: function(aInstall) {
+  onDownloadProgress: function gInstallingPage_onDownloadProgress(aInstall) {
     var downloadProgress = document.getElementById("downloadProgress");
     downloadProgress.value = Math.ceil(100 * aInstall.progress / aInstall.maxProgress);
   },
 
-  onDownloadEnded: function(aInstall) {
+  onDownloadEnded: function gInstallingPage_onDownloadEnded(aInstall) {
   },
 
-  onDownloadFailed: function(aInstall) {
+  onDownloadFailed: function gInstallingPage_onDownloadFailed(aInstall) {
     this._errors.push(aInstall);
 
     this.startNextInstall();
   },
 
-  onInstallStarted: function(aInstall) {
+  onInstallStarted: function gInstallingPage_onInstallStarted(aInstall) {
     var strings = document.getElementById("updateStrings");
     var label = strings.getFormattedString("installingPrefix", [aInstall.name]);
     var actionItem = document.getElementById("actionItem");
     actionItem.value = label;
   },
 
-  onInstallEnded: function(aInstall) {
+  onInstallEnded: function gInstallingPage_onInstallEnded(aInstall, aAddon) {
     // Remember that this add-on was updated during startup
     AddonManagerPrivate.addStartupChange(AddonManager.STARTUP_CHANGE_CHANGED,
-                                         aInstall.id);
+                                         aAddon.id);
 
     this.startNextInstall();
   },
 
-  onInstallFailed: function(aInstall) {
+  onInstallFailed: function gInstallingPage_onInstallFailed(aInstall) {
     this._errors.push(aInstall);
 
     this.startNextInstall();
@@ -452,7 +458,7 @@ var gInstallingPage = {
 };
 
 var gInstallErrorsPage = {
-  onPageShow: function ()
+  onPageShow: function gInstallErrorsPage_onPageShow()
   {
     gUpdateWizard.setButtonLabels(null, true, null, true, null, true);
     document.documentElement.getButton("finish").focus();
@@ -462,7 +468,7 @@ var gInstallErrorsPage = {
 // Displayed when there are incompatible add-ons and the xpinstall.enabled
 // pref is false and locked.
 var gAdminDisabledPage = {
-  onPageShow: function ()
+  onPageShow: function gAdminDisabledPage_onPageShow()
   {
     gUpdateWizard.setButtonLabels(null, true, null, true,
                                   "cancelButtonText", true);
@@ -473,7 +479,7 @@ var gAdminDisabledPage = {
 // Displayed when selected add-on updates have been installed without error.
 // There can still be add-ons that are not compatible and don't have an update.
 var gFinishedPage = {
-  onPageShow: function ()
+  onPageShow: function gFinishedPage_onPageShow()
   {
     gUpdateWizard.setButtonLabels(null, true, null, true, null, true);
     document.documentElement.getButton("finish").focus();
@@ -492,7 +498,7 @@ var gFinishedPage = {
 // Displayed when there are incompatible add-ons and there are no available
 // updates.
 var gNoUpdatesPage = {
-  onPageShow: function (aEvent)
+  onPageShow: function gNoUpdatesPage_onPageLoad(aEvent)
   {
     gUpdateWizard.setButtonLabels(null, true, null, true, null, true);
     if (gUpdateWizard.shouldSuggestAutoChecking) {
